@@ -1,20 +1,43 @@
+"""Defines aspects for Items.
+
+Aspects form intermediate items between tools and items.
 """
-Defines aspects for Items. Aspects form intermediate items between tools
-and items.
+import warnings
+from functools import singledispatch as real_singledispatch
 
-Note: This module uses Phillip J. Eby's simplegeneric module. This module
-transforms the generic class (used as fall-back) to a generic function. In
-order to inherit from this class you should inherit from Class.default.
-The simplegeneric module is dispatching opnly based on the first argument.
-For Gaphas that's enough.
-"""
+from gi.repository import Gdk
 
-import gtk.gdk
-from simplegeneric import generic
-from gaphas.item import Item, Element
+from gaphas.item import Element, Item
 
 
-class ItemFinder(object):
+def singledispatch(func):
+    """
+    Wrapper around singledispatch(), with an extra compatibility function
+    so code will not break when upgrading from 1.0 to 1.1.
+    """
+    wrapped = real_singledispatch(func)
+
+    def when_type(*types):
+        if not types:
+            raise TypeError("should provide at least one type")
+        warnings.warn(
+            "when_type: is deprecated, use `register` instead",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
+
+        def wrapper_for_types(func):
+            for cls in types:
+                wrapped.register(cls, func)
+            return func
+
+        return wrapper_for_types
+
+    wrapped.when_type = when_type
+    return wrapped
+
+
+class ItemFinder:
     """
     Find an item on the canvas.
     """
@@ -27,10 +50,10 @@ class ItemFinder(object):
         return item or self.view.get_item_at_point(pos)
 
 
-Finder = generic(ItemFinder)
+Finder = singledispatch(ItemFinder)
 
 
-class ItemSelection(object):
+class ItemSelection:
     """
     A role for items. When dealing with selection.
 
@@ -53,10 +76,10 @@ class ItemSelection(object):
         self.view.unselect_item(self.item)
 
 
-Selection = generic(ItemSelection)
+Selection = singledispatch(ItemSelection)
 
 
-class ItemInMotion(object):
+class ItemInMotion:
     """
     Aspect for dealing with motion on an item.
 
@@ -91,10 +114,10 @@ class ItemInMotion(object):
         pass
 
 
-InMotion = generic(ItemInMotion)
+InMotion = singledispatch(ItemInMotion)
 
 
-class ItemHandleFinder(object):
+class ItemHandleFinder:
     """
     Deals with the task of finding handles.
     """
@@ -107,10 +130,10 @@ class ItemHandleFinder(object):
         return self.view.get_handle_at_point(pos)
 
 
-HandleFinder = generic(ItemHandleFinder)
+HandleFinder = singledispatch(ItemHandleFinder)
 
 
-class ItemHandleSelection(object):
+class ItemHandleSelection:
     """
     Deal with selection of the handle.
     """
@@ -127,30 +150,28 @@ class ItemHandleSelection(object):
         pass
 
 
-HandleSelection = generic(ItemHandleSelection)
+HandleSelection = singledispatch(ItemHandleSelection)
 
 
-@HandleSelection.when_type(Element)
+@HandleSelection.register(Element)
 class ElementHandleSelection(ItemHandleSelection):
-    CURSORS = (
-            gtk.gdk.Cursor(gtk.gdk.TOP_LEFT_CORNER),
-            gtk.gdk.Cursor(gtk.gdk.TOP_RIGHT_CORNER),
-            gtk.gdk.Cursor(gtk.gdk.BOTTOM_RIGHT_CORNER),
-            gtk.gdk.Cursor(gtk.gdk.BOTTOM_LEFT_CORNER) )
+    CURSORS = ("nw-resize", "ne-resize", "se-resize", "sw-resize")
 
     def select(self):
         index = self.item.handles().index(self.handle)
         if index < 4:
-            self.view.window.set_cursor(self.CURSORS[index])
+            display = self.view.get_display()
+            cursor = Gdk.Cursor.new_from_name(display, self.CURSORS[index])
+            self.view.get_window().set_cursor(cursor)
 
     def unselect(self):
-        from view import DEFAULT_CURSOR
-        cursor = gtk.gdk.Cursor(DEFAULT_CURSOR)
-        self.view.window.set_cursor(cursor)
+        from .view import DEFAULT_CURSOR
+
+        cursor = Gdk.Cursor(DEFAULT_CURSOR)
+        self.view.get_window().set_cursor(cursor)
 
 
-
-class ItemHandleInMotion(object):
+class ItemHandleInMotion:
     """
     Move a handle (role is applied to the handle)
     """
@@ -206,8 +227,9 @@ class ItemHandleInMotion(object):
         if not handle.connectable:
             return None
 
-        connectable, port, glue_pos = \
-                view.get_port_at_point(pos, distance=distance, exclude=(item,))
+        connectable, port, glue_pos = view.get_port_at_point(
+            pos, distance=distance, exclude=(item,)
+        )
 
         # check if item and found item can be connected on closest port
         if port is not None:
@@ -217,20 +239,23 @@ class ItemHandleInMotion(object):
             sink = ConnectionSink(connectable, port)
 
             if connector.allow(sink):
-                # transform coordinates from view space to the item space and
-                # update position of item's handle
+                # transform coordinates from view space to the item
+                # space and update position of item's handle
                 v2i = view.get_matrix_v2i(item).transform_point
                 handle.pos = v2i(*glue_pos)
                 return sink
         return None
 
 
-HandleInMotion = generic(ItemHandleInMotion)
+HandleInMotion = singledispatch(ItemHandleInMotion)
 
 
-class ItemConnector(object):
+class ItemConnector:
+    """Connect or disconnect an item's handle to another item or port.
 
-    GLUE_DISTANCE = 10 # Glue distance in view points
+    """
+
+    GLUE_DISTANCE = 10  # Glue distance in view points
 
     def __init__(self, item, handle):
         self.item = item
@@ -255,8 +280,8 @@ class ItemConnector(object):
         """
         Connect the handle to a sink (item, port).
 
-        Note that connect() also takes care of disconnecting in case a handle
-        is reattached to another element.
+        Note that connect() also takes care of disconnecting in case a
+        handle is reattached to another element.
         """
 
         cinfo = self.item.canvas.get_connection(self.handle)
@@ -272,11 +297,10 @@ class ItemConnector(object):
 
         self.connect_handle(sink)
 
-
     def connect_handle(self, sink, callback=None):
         """
-        Create constraint between handle of a line and port of connectable
-        item.
+        Create constraint between handle of a line and port of
+        connectable item.
 
         :Parameters:
          sink
@@ -290,9 +314,9 @@ class ItemConnector(object):
 
         constraint = sink.port.constraint(canvas, item, handle, sink.item)
 
-        canvas.connect_item(item, handle, sink.item, sink.port,
-            constraint, callback=callback)
-
+        canvas.connect_item(
+            item, handle, sink.item, sink.port, constraint, callback=callback
+        )
 
     def disconnect(self):
         """
@@ -301,12 +325,15 @@ class ItemConnector(object):
         self.item.canvas.disconnect_item(self.item, self.handle)
 
 
-Connector = generic(ItemConnector)
+Connector = singledispatch(ItemConnector)
 
 
-class ItemConnectionSink(object):
-    """
-    This role should be applied to items that is connected to.
+class ItemConnectionSink:
+    """Makes an item a sink.
+
+    A sink is another item that an item's handle is connected to like a
+    connectable item or port.
+
     """
 
     def __init__(self, item, port):
@@ -330,14 +357,15 @@ class ItemConnectionSink(object):
         return port
 
 
-ConnectionSink = generic(ItemConnectionSink)
+ConnectionSink = singledispatch(ItemConnectionSink)
 
 
-##
-## Painter aspects
-##
+#
+# Painter aspects
+#
 
-class ItemPaintFocused(object):
+
+class ItemPaintFocused:
     """
     Paints on top of all items, just for the focused item and only
     when it's hovered (see gaphas.painter.FocusedItemPainter)
@@ -351,7 +379,4 @@ class ItemPaintFocused(object):
         pass
 
 
-PaintFocused = generic(ItemPaintFocused)
-
-
-# vim:sw=4:et:ai
+PaintFocused = singledispatch(ItemPaintFocused)
