@@ -1,8 +1,8 @@
 """
-This module contains several flavors of constraint classes.
-Each has a method `Constraint.solve_for(name)` and a method
-`Constraint.mark_dirty(v)`. These methods are used by the constraint solver
-(`solver.Solver`) to set the variables.
+This module contains several flavors of constraint classes.  Each has
+a method `Constraint.solve_for(name)` and a method
+`Constraint.mark_dirty(v)`. These methods are used by the constraint
+solver (`solver.Solver`) to set the variables.
 
 Variables should be of type `solver.Variable`.
 
@@ -12,7 +12,7 @@ examples of their usage.
 EqualsConstraint
     Make 'a' and 'b' equal.
 LessThanConstraint
-    Ensure one variable stays smaller than the other.
+    Ensure one variable stays smaller than another.
 CenterConstraint
     Ensures a Variable is kept between two other variables.
 EquationConstraint
@@ -20,55 +20,53 @@ EquationConstraint
 BalanceConstraint
     Keeps three variables in line, maintaining a specific ratio.
 LineConstraint
-    Solves the equation where a line is connected to a line or side at a
-    specific point.
+    Solves the equation where a line is connected to a line or side at
+    a specific point.
 
-New constraint class should derive from Constraint class abstract class and
-implement `Constraint.solve_for(Variable)` method to update a variable with
-appropriate value.
+New constraint class should derive from Constraint class abstract
+class and implement `Constraint.solve_for(Variable)` method to update
+a variable with appropriate value.
 """
-
-from __future__ import division
-import operator
 import math
-from solver import Projection
-
-
-__version__ = "$Revision$"
-# $HeadURL$
 
+from gaphas.solver import Projection
 
 # is simple abs(x - y) > EPSILON enough for canvas needs?
 EPSILON = 1e-6
 
+
 def _update(variable, value):
     if abs(variable.value - value) > EPSILON:
         variable.value = value
 
 
-class Constraint(object):
+class Constraint:
     """
     Constraint base class.
 
     - _variables - list of all variables
     - _weakest   - list of weakest variables
     """
+
     disabled = False
 
     def __init__(self, *variables):
         """
-        Create new constraint, register all variables, and find weakest
-        variables.
+        Create new constraint, register all variables, and find
+        weakest variables.
+
+        Any value can be added. It is assumed to be a variable if it
+        has a 'strength' attribute.
         """
         self._variables = []
         for v in variables:
-            self._variables.append(v)
+            if hasattr(v, "strength"):
+                self._variables.append(v)
 
         self.create_weakest_list()
 
         # Used by the Solver for efficiency
-        self._solver_has_projections = False 
-
+        self._solver_has_projections = False
 
     def create_weakest_list(self):
         """
@@ -78,7 +76,6 @@ class Constraint(object):
         strength = min(v.strength for v in self._variables)
         self._weakest = [v for v in self._variables if v.strength == strength]
 
-
     def variables(self):
         """
         Return an iterator which iterates over the variables that are
@@ -86,20 +83,18 @@ class Constraint(object):
         """
         return self._variables
 
-
     def weakest(self):
         """
-        Return the weakest variable. The weakest variable should be always
-        as first element of Constraint._weakest list.
+        Return the weakest variable. The weakest variable should be
+        always as first element of Constraint._weakest list.
         """
         return self._weakest[0]
 
-
     def mark_dirty(self, v):
         """
         Mark variable v dirty and if possible move it to the end of
-        Constraint._weakest list to maintain weakest variable invariants
-        (see gaphas.solver module documentation).
+        Constraint._weakest list to maintain weakest variable
+        invariants (see gaphas.solver module documentation).
         """
         weakest = self.weakest()
         # Fast lane:
@@ -118,6 +113,16 @@ class Constraint(object):
                 self._weakest.append(p)
                 return
 
+    def solve(self):
+        """
+        Solve the constraint. This is done by determining the weakest
+        variable and calling solve_for() for that variable. The
+        weakest variable is always in the set of variables with the
+        weakest strength. The least recently changed variable is
+        considered the weakest.
+        """
+        wvar = self.weakest()
+        self.solve_for(wvar)
 
     def solve_for(self, var):
         """
@@ -127,12 +132,14 @@ class Constraint(object):
         raise NotImplemented
 
 
-
 class EqualsConstraint(Constraint):
     """
-    Constraint, which ensures that two arguments ``a`` and ``b`` are equal,
+    Constraint, which ensures that two arguments ``a`` and ``b`` are equal:
+
+        a + delta = b
+
     for example
-    >>> from solver import Variable
+    >>> from gaphas.solver import Variable
     >>> a, b = Variable(1.0), Variable(2.0)
     >>> eq = EqualsConstraint(a, b)
     >>> eq.solve_for(a)
@@ -145,28 +152,32 @@ class EqualsConstraint(Constraint):
     """
 
     def __init__(self, a=None, b=None, delta=0.0):
-        super(EqualsConstraint, self).__init__(a, b)
+        super().__init__(a, b, delta)
         self.a = a
         self.b = b
-        self._delta = delta
-
+        self.delta = delta
 
     def solve_for(self, var):
-        assert var in (self.a, self.b)
-
-        _update(*((var is self.a) and \
-                (self.a, self.b.value + self._delta) or \
-                (self.b, self.a.value + self._delta)))
+        assert var in (self.a, self.b, self.delta)
 
+        _update(
+            *(
+                (var is self.a)
+                and (self.a, self.b.value - self.delta)
+                or (var is self.b)
+                and (self.b, self.a.value + self.delta)
+                or (self.delta, self.b.value - self.a.value)
+            )
+        )
 
 
 class CenterConstraint(Constraint):
     """
     Simple Constraint, takes three arguments: 'a', 'b' and center.
-    When solved, the constraint ensures 'center' is located in the middle
-    of 'a' and 'b'.
+    When solved, the constraint ensures 'center' is located in the
+    middle of 'a' and 'b'.
 
-    >>> from solver import Variable
+    >>> from gaphas.solver import Variable
     >>> a, b, center = Variable(1.0), Variable(3.0), Variable()
     >>> eq = CenterConstraint(a, b, center)
     >>> eq.solve_for(a)
@@ -183,12 +194,11 @@ class CenterConstraint(Constraint):
     """
 
     def __init__(self, a=None, b=None, center=None):
-        super(CenterConstraint, self).__init__(a, b, center)
+        super().__init__(a, b, center)
         self.a = a
         self.b = b
         self.center = center
 
-
     def solve_for(self, var):
         assert var in (self.a, self.b, self.center)
 
@@ -196,14 +206,14 @@ class CenterConstraint(Constraint):
         _update(self.center, v)
 
 
-
 class LessThanConstraint(Constraint):
     """
-    Ensure ``smaller`` is less than ``bigger``. The variable that is passed
-    as to-be-solved is left alone (cause it is the variable that has not
-    been moved lately). Instead the other variable is solved.
+    Ensure ``smaller`` is less than ``bigger``. The variable that is
+    passed as to-be-solved is left alone (cause it is the variable
+    that has not been moved lately). Instead the other variable is
+    solved.
 
-    >>> from solver import Variable
+    >>> from gaphas.solver import Variable
     >>> a, b = Variable(3.0), Variable(2.0)
     >>> lt = LessThanConstraint(smaller=a, bigger=b)
     >>> lt.solve_for(a)
@@ -224,33 +234,34 @@ class LessThanConstraint(Constraint):
     """
 
     def __init__(self, smaller=None, bigger=None, delta=0.0):
-        super(LessThanConstraint, self).__init__(smaller, bigger)
+        super().__init__(smaller, bigger, delta)
         self.smaller = smaller
         self.bigger = bigger
         self.delta = delta
 
-
     def solve_for(self, var):
         if self.smaller.value > self.bigger.value - self.delta:
             if var is self.smaller:
                 self.bigger.value = self.smaller.value + self.delta
             elif var is self.bigger:
                 self.smaller.value = self.bigger.value - self.delta
-
+            elif var is self.delta:
+                self.delta.value = self.bigger.value - self.smaller.value
 
 
 # Constants for the EquationConstraint
-ITERLIMIT = 1000        # iteration limit
+ITERLIMIT = 1000  # iteration limit
+
 
 class EquationConstraint(Constraint):
     """
     Equation solver using attributes and introspection.
 
-    Takes a function, named arg value (opt.) and returns a Constraint object
-    Calling EquationConstraint.solve_for will solve the equation for
-    variable ``arg``, so that the outcome is 0.
+    Takes a function, named arg value (opt.) and returns a Constraint
+    object Calling EquationConstraint.solve_for will solve the
+    equation for variable ``arg``, so that the outcome is 0.
 
-    >>> from solver import Variable
+    >>> from gaphas.solver import Variable
     >>> a, b, c = Variable(), Variable(4), Variable(5)
     >>> cons = EquationConstraint(lambda a, b, c: a + b - c, a=a, b=b, c=c)
     >>> cons.solve_for(a)
@@ -263,25 +274,24 @@ class EquationConstraint(Constraint):
 
     From: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/303396
     """
-    
+
     def __init__(self, f, **args):
-        super(EquationConstraint, self).__init__(*args.values())
+        super().__init__(*list(args.values()))
         self._f = f
         self._args = {}
         # see important note on order of operations in __setattr__ below.
-        for arg in f.func_code.co_varnames[0:f.func_code.co_argcount]:
+        for arg in f.__code__.co_varnames[0 : f.__code__.co_argcount]:
             self._args[arg] = None
         self._set(**args)
 
-
     def __repr__(self):
-        argstring = ', '.join(['%s=%s' % (arg, str(value)) for (arg, value) in
-                             self._args.items()])
+        argstring = ", ".join(
+            [f"{arg}={value}" for (arg, value) in list(self._args.items())]
+        )
         if argstring:
-            return 'EquationConstraint(%s, %s)' % (self._f.func_code.co_name, argstring)
+            return f"EquationConstraint({self._f.__code__.co_name}, {argstring})"
         else:
-            return 'EquationConstraint(%s)' % self._f.func_code.co_name
-
+            return f"EquationConstraint({self._f.__code__.co_name})"
 
     def __getattr__(self, name):
         """
@@ -290,7 +300,6 @@ class EquationConstraint(Constraint):
         self._args[name]
         return self.solve_for(name)
 
-
     def __setattr__(self, name, value):
         """
         Sets function argument values.
@@ -299,17 +308,16 @@ class EquationConstraint(Constraint):
         # be added to self.__dict__.  This is a good thing as it throws
         # an exception if you try to assign to an arg which is inappropriate
         # for the function in the solver.
-        if self.__dict__.has_key('_args'):
+        if "_args" in self.__dict__:
             if name in self._args:
                 self._args[name] = value
             elif name in self.__dict__:
                 self.__dict__[name] = value
             else:
-                raise KeyError, name
+                raise KeyError(name)
         else:
             object.__setattr__(self, name, value)
 
-
     def _set(self, **args):
         """
         Sets values of function arguments.
@@ -318,27 +326,26 @@ class EquationConstraint(Constraint):
             self._args[arg]  # raise exception if arg not in _args
             setattr(self, arg, args[arg])
 
-
     def solve_for(self, var):
         """
         Solve this constraint for the variable named 'arg' in the
         constraint.
         """
         args = {}
-        for nm, v in self._args.items():
+        for nm, v in list(self._args.items()):
             args[nm] = v.value
-            if v is var: arg = nm
-        v = self._solve_for(arg, args) 
+            if v is var:
+                arg = nm
+        v = self._solve_for(arg, args)
         if var.value != v:
             var.value = v
 
-
     def _solve_for(self, arg, args):
         """
         Newton's method solver
         """
-        #args = self._args
-        close_runs = 10   # after getting close, do more passes
+        # args = self._args
+        close_runs = 10  # after getting close, do more passes
         if args[arg]:
             x0 = args[arg]
         else:
@@ -346,36 +353,38 @@ class EquationConstraint(Constraint):
         if x0 == 0:
             x1 = 1
         else:
-            x1 = x0*1.1
+            x1 = x0 * 1.1
+
         def f(x):
             """function to solve"""
             args[arg] = x
             return self._f(**args)
+
         fx0 = f(x0)
         n = 0
-        while 1:                    # Newton's method loop here
+        while True:  # Newton's method loop here
             fx1 = f(x1)
             if fx1 == 0 or x1 == x0:  # managed to nail it exactly
                 break
-            if abs(fx1-fx0) < EPSILON:    # very close
+            if abs(fx1 - fx0) < EPSILON:  # very close
                 close_flag = True
-                if close_runs == 0:       # been close several times
+                if close_runs == 0:  # been close several times
                     break
                 else:
-                    close_runs -= 1       # try some more
+                    close_runs -= 1  # try some more
             else:
                 close_flag = False
             if n > ITERLIMIT:
-                print "Failed to converge; exceeded iteration limit"
+                print("Failed to converge; exceeded iteration limit")
                 break
             slope = (fx1 - fx0) / (x1 - x0)
             if slope == 0:
                 if close_flag:  # we're close but have zero slope, finish
                     break
                 else:
-                    print 'Zero slope and not close enough to solution'
+                    print("Zero slope and not close enough to solution")
                     break
-            x2 = x0 - fx0 / slope           # New 'x1'
+            x2 = x0 - fx0 / slope  # New 'x1'
             fx0 = fx1
             x0 = x1
             x1 = x2
@@ -383,15 +392,14 @@ class EquationConstraint(Constraint):
         return x1
 
 
-
 class BalanceConstraint(Constraint):
     """
-    Ensure that a variable ``v`` is between values specified by ``band``
-    and in distance proportional from ``band[0]``.
+    Ensure that a variable ``v`` is between values specified by
+    ``band`` and in distance proportional from ``band[0]``.
 
     Consider
 
-    >>> from solver import Variable, WEAK
+    >>> from gaphas.solver import Variable, WEAK
     >>> a, b, c = Variable(2.0), Variable(3.0), Variable(2.3, WEAK)
     >>> bc = BalanceConstraint(band=(a,b), v=c)
     >>> c.value = 2.4
@@ -414,7 +422,7 @@ class BalanceConstraint(Constraint):
     """
 
     def __init__(self, band=None, v=None, balance=None):
-        super(BalanceConstraint, self).__init__(band[0], band[1], v)
+        super().__init__(band[0], band[1], v)
         self.band = band
         self.balance = balance
         self.v = v
@@ -422,15 +430,10 @@ class BalanceConstraint(Constraint):
         if self.balance is None:
             self.update_balance()
 
-
     def update_balance(self):
         b1, b2 = self.band
         w = b2 - b1
-        if w != 0:
-            self.balance = (self.v - b1) / w
-        else:
-            self.balance = 0
-
+        self.balance = (self.v - b1) / w if w != 0 else 0
 
     def solve_for(self, var):
         b1, b2 = self.band
@@ -439,7 +442,6 @@ class BalanceConstraint(Constraint):
         _update(var, value)
 
 
-
 class LineConstraint(Constraint):
     """
     Ensure a point is kept on a line.
@@ -450,13 +452,14 @@ class LineConstraint(Constraint):
     """
 
     def __init__(self, line, point):
-        super(LineConstraint, self).__init__(line[0][0], line[0][1], line[1][0], line[1][1], point[0], point[1])
+        super().__init__(
+            line[0][0], line[0][1], line[1][0], line[1][1], point[0], point[1]
+        )
 
         self._line = line
         self._point = point
         self.update_ratio()
 
-
     def update_ratio(self):
         """
         >>> from gaphas.solver import Variable
@@ -465,12 +468,12 @@ class LineConstraint(Constraint):
         >>> lc = LineConstraint(line=line, point=point)
         >>> lc.update_ratio()
         >>> lc.ratio_x, lc.ratio_y
-        (0.5, 0.20000000000000001)
+        (0.5, 0.2)
         >>> line[1][0].value = 40
         >>> line[1][1].value = 30
         >>> lc.solve_for(point[0])
         >>> lc.ratio_x, lc.ratio_y
-        (0.5, 0.20000000000000001)
+        (0.5, 0.2)
         >>> point
         (Variable(20, 20), Variable(6, 20))
         """
@@ -487,15 +490,13 @@ class LineConstraint(Constraint):
         except ZeroDivisionError:
             self.ratio_y = 0.0
 
-        
     def solve_for(self, var=None):
         self._solve()
 
-
     def _solve(self):
         """
         Solve the equation for the connected_handle.
-        
+
         >>> from gaphas.solver import Variable
         >>> line = (Variable(0), Variable(0)), (Variable(30), Variable(20))
         >>> point = (Variable(15), Variable(4))
@@ -521,7 +522,6 @@ class LineConstraint(Constraint):
         _update(py, y)
 
 
-
 class PositionConstraint(Constraint):
     """
     Ensure that point is always in origin position.
@@ -532,13 +532,11 @@ class PositionConstraint(Constraint):
     """
 
     def __init__(self, origin, point):
-        super(PositionConstraint, self).__init__(origin[0], origin[1],
-                point[0], point[1])
+        super().__init__(origin[0], origin[1], point[0], point[1])
 
         self._origin = origin
         self._point = point
 
-        
     def solve_for(self, var=None):
         """
         Ensure that point's coordinates are the same as coordinates of the
@@ -549,11 +547,10 @@ class PositionConstraint(Constraint):
         _update(self._point[1], y)
 
 
-
 class LineAlignConstraint(Constraint):
     """
-    Ensure a point is kept on a line in position specified by align and padding
-    information.
+    Ensure a point is kept on a line in position specified by align
+    and padding information.
 
     Align is specified as a number between 0 and 1, for example
      0
@@ -578,14 +575,15 @@ class LineAlignConstraint(Constraint):
     """
 
     def __init__(self, line, point, align=0.5, delta=0.0):
-        super(LineAlignConstraint, self).__init__(line[0][0], line[0][1], line[1][0], line[1][1], point[0], point[1])
+        super().__init__(
+            line[0][0], line[0][1], line[1][0], line[1][1], point[0], point[1]
+        )
 
         self._line = line
         self._point = point
         self._align = align
         self._delta = delta
 
-        
     def solve_for(self, var=None):
         sx, sy = self._line[0]
         ex, ey = self._line[1]
@@ -597,6 +595,3 @@ class LineAlignConstraint(Constraint):
 
         _update(px, x)
         _update(py, y)
-
-
-# vim:sw=4:et:ai
