@@ -1,67 +1,66 @@
 """
 Custom decorators.
 """
-
-__version__ = "$Revision$"
-# $HeadURL$
-
 import threading
-import gobject
-from gobject import PRIORITY_HIGH, PRIORITY_HIGH_IDLE, PRIORITY_DEFAULT, \
-        PRIORITY_DEFAULT_IDLE, PRIORITY_LOW
 
+import gi
+
+gi.require_version("Gtk", "3.0")
+from gi.repository import Gtk, GLib
 
 DEBUG_ASYNC = False
 
 
-class async(object):
+class AsyncIO:
     """
-    Instead of calling the function, schedule an idle handler at a given
-    priority. This requires the async'ed method to be called from within
-    the GTK main loop. Otherwise the method is executed directly.
+    Instead of calling the function, schedule an idle handler at a
+    given priority. This requires the async'ed method to be called
+    from within the GTK main loop. Otherwise the method is executed
+    directly.
 
     Note:
         the current implementation of async single mode only works for
         methods, not functions.
 
-    Calling the async function from outside the gtk main loop will yield
-    imediate execution:
+    Calling the async function from outside the gtk main loop will
+    yield immediate execution:
 
     async just works on functions (as long as ``single=False``):
 
-    >>> a = async()(lambda: 'Hi')
+    >>> a = AsyncIO()(lambda: 'Hi')
     >>> a()
     'Hi'
 
     Simple method:
-    
+
     >>> class A(object):
-    ...     @async(single=False, priority=gobject.PRIORITY_HIGH)
+    ...     @AsyncIO(single=False, priority=GLib.PRIORITY_HIGH)
     ...     def a(self):
-    ...         print 'idle-a', gobject.main_depth()
-    
-    Methods can also set single mode to True (the method is only scheduled one).
+    ...         print('idle-a', GLib.main_depth())
+
+    Methods can also set single mode to True (the method is only
+    scheduled once).
 
     >>> class B(object):
-    ...     @async(single=True)
+    ...     @AsyncIO(single=True)
     ...     def b(self):
-    ...         print 'idle-b', gobject.main_depth()
+    ...         print('idle-b', GLib.main_depth())
 
     Also a timeout property can be provided:
 
     >>> class C(object):
-    ...     @async(timeout=50)
+    ...     @AsyncIO(timeout=50)
     ...     def c1(self):
-    ...         print 'idle-c1', gobject.main_depth()
-    ...     @async(single=True, timeout=60)
+    ...         print('idle-c1', GLib.main_depth())
+    ...     @AsyncIO(single=True, timeout=60)
     ...     def c2(self):
-    ...         print 'idle-c2', gobject.main_depth()
+    ...         print('idle-c2', GLib.main_depth())
 
-    This is a helper function used to test classes A and B from within the GTK+
-    main loop:
+    This is a helper function used to test classes A and B from within
+    the GTK+ main loop:
 
     >>> def delayed():
-    ...     print 'before'
+    ...     print("before")
     ...     a = A()
     ...     b = B()
     ...     c = C()
@@ -75,12 +74,12 @@ class async(object):
     ...     b.b()
     ...     a.a()
     ...     b.b()
-    ...     print 'after'
-    ...     gobject.timeout_add(100, gtk.main_quit)
-    >>> gobject.timeout_add(1, delayed) > 0 # timeout id may vary
+    ...     print("after")
+    ...     GLib.timeout_add(100, Gtk.main_quit)
+    >>> GLib.timeout_add(1, delayed) > 0 # timeout id may vary
     True
-    >>> import gtk
-    >>> gtk.main()
+    >>> from gi.repository import Gtk
+    >>> Gtk.main()
     before
     after
     idle-a 1
@@ -91,55 +90,57 @@ class async(object):
     idle-c1 1
     idle-c2 1
 
-    As you can see, although ``b.b()`` has been called three times, it's only
-    executed once.
+    As you can see, although ``b.b()`` has been called three times,
+    it's only executed once.
     """
 
-    def __init__(self, single=False, timeout=0, priority=gobject.PRIORITY_DEFAULT):
+    def __init__(self, single=False, timeout=0, priority=GLib.PRIORITY_DEFAULT):
         self.single = single
         self.timeout = timeout
         self.priority = priority
 
     def source(self, func):
         timeout = self.timeout
-        if timeout > 0:
-            s = gobject.Timeout(timeout)
-        else:
-            s = gobject.Idle()
+        s = GLib.Timeout(timeout) if timeout > 0 else GLib.Idle()
         s.set_callback(func)
         s.priority = self.priority
         return s
 
     def __call__(self, func):
-        async_id = '_async_id_%s' % func.__name__
-        source = self.source
+        async_id = f"_async_id_{func.__name__}"
 
         def wrapper(*args, **kwargs):
             global getattr, setattr, delattr
-            # execute directly if we're not in the main loop.
-            if gobject.main_depth() == 0:
+            # execute directly if we're not in the main loop
+            if GLib.main_depth() == 0:
                 return func(*args, **kwargs)
             elif not self.single:
-                def async_wrapper():
-                    if DEBUG_ASYNC: print 'async:', func, args, kwargs
+
+                def async_wrapper(*aargs):
+                    if DEBUG_ASYNC:
+                        print("async:", func, args, kwargs)
                     func(*args, **kwargs)
-                source(async_wrapper).attach()
+
+                self.source(async_wrapper).attach()
             else:
                 # Idle handlers should be registered per instance
                 holder = args[0]
                 try:
                     if getattr(holder, async_id):
                         return
-                except AttributeError, e:
-                    def async_wrapper():
-                        if DEBUG_ASYNC: print 'async:', func, args, kwargs
+                except AttributeError as e:
+
+                    def async_wrapper(*aargs):
+                        if DEBUG_ASYNC:
+                            print("async:", func, args, kwargs)
                         try:
                             func(*args, **kwargs)
                         finally:
                             delattr(holder, async_id)
                         return False
 
-                    setattr(holder, async_id, source(async_wrapper).attach())
+                    setattr(holder, async_id, self.source(async_wrapper).attach())
+
         return wrapper
 
 
@@ -150,7 +151,7 @@ def nonrecursive(func):
     >>> class A(object):
     ...     @nonrecursive
     ...     def a(self, x=1):
-    ...         print x
+    ...         print(x)
     ...         self.a(x+1)
     >>> A().a()
     1
@@ -158,19 +159,21 @@ def nonrecursive(func):
     1
     """
     m = threading.Lock()
+
     def wrapper(*args, **kwargs):
         """
-        Decorate function with a mutex that prohibits recursice execution.
+        Decorate function with a mutex that prohibits recursive execution.
         """
         if m.acquire(False):
             try:
                 return func(*args, **kwargs)
             finally:
                 m.release()
+
     return wrapper
 
 
-class recursive(object):
+class recursive:
     """
     This decorator limits the recursion for a specific function
 
@@ -201,7 +204,5 @@ class recursive(object):
                     return func(*args, **kwargs)
                 finally:
                     func._recursion_level -= 1
+
         return wrapper
-
-
-# vim:sw=4:et:ai
