@@ -7,18 +7,14 @@ intersections).
 A point is represented as a tuple `(x, y)`.
 
 """
-
-__version__ = "$Revision$"
-# $HeadURL$
-
 from math import sqrt
 
 
-class Rectangle(object):
+class Rectangle:
     """
     Python Rectangle implementation. Rectangles can be added (union),
-    substituted (intersection) and points and rectangles can be tested to
-    be in the rectangle.
+    substituted (intersection) and points and rectangles can be tested
+    to be in the rectangle.
 
     >>> r1= Rectangle(1,1,5,5)
     >>> r2 = Rectangle(3,3,6,7)
@@ -61,16 +57,18 @@ class Rectangle(object):
         """
         """
         width = x1 - self.x
-        if width < 0: width = 0
+        if width < 0:
+            width = 0
         self.width = width
-        
+
     x1 = property(lambda s: s.x + s.width, _set_x1)
 
     def _set_y1(self, y1):
         """
         """
         height = y1 - self.y
-        if height < 0: height = 0
+        if height < 0:
+            height = 0
         self.height = height
 
     y1 = property(lambda s: s.y + s.height, _set_y1)
@@ -86,15 +84,15 @@ class Rectangle(object):
         self.y -= delta
         self.width += delta * 2
         self.height += delta * 2
-        
+
     def __repr__(self):
         """
         >>> Rectangle(5,7,20,25)
         Rectangle(5, 7, 20, 25)
         """
         if self:
-            return '%s(%g, %g, %g, %g)' % (self.__class__.__name__, self.x, self.y, self.width, self.height)
-        return '%s()' % self.__class__.__name__
+            return f"({self.__class__.__name__}, {self.x}, {self.y}, {self.width}, {self.height})"
+        return f"{self.__class__.__name__}()"
 
     def __iter__(self):
         """
@@ -110,7 +108,7 @@ class Rectangle(object):
         """
         return (self.x, self.y, self.width, self.height)[index]
 
-    def __nonzero__(self):
+    def __bool__(self):
         """
         >>> r=Rectangle(1,2,3,4)
         >>> if r: 'yes'
@@ -119,13 +117,15 @@ class Rectangle(object):
         >>> if r: 'no'
         """
         return self.width > 0 and self.height > 0
-    
+
     def __eq__(self, other):
-        return (type(self) is type(other)) \
-                and self.x == other.x \
-                and self.y == other.y \
-                and self.width == other.width \
-                and self.height == self.height
+        return (
+            (isinstance(self, type(other)))
+            and self.x == other.x
+            and self.y == other.y
+            and self.width == other.width
+            and self.height == self.height
+        )
 
     def __add__(self, obj):
         """
@@ -157,7 +157,9 @@ class Rectangle(object):
         try:
             x, y, width, height = obj
         except ValueError:
-            raise TypeError, "Can only add Rectangle or tuple (x, y, width, height), not %s." % repr(obj)
+            raise TypeError(
+                f"Can only add Rectangle or tuple (x, y, width, height), not {repr(obj)}."
+            )
         x1, y1 = x + width, y + height
         if self:
             ox1, oy1 = self.x + self.width, self.y + self.height
@@ -192,12 +194,14 @@ class Rectangle(object):
         >>> r -= 'aap'
         Traceback (most recent call last):
           ...
-        TypeError: Can only substract Rectangle or tuple (x, y, width, height), not 'aap'.
+        TypeError: Can only subtract Rectangle or tuple (x, y, width, height), not 'aap'.
         """
         try:
             x, y, width, height = obj
         except ValueError:
-            raise TypeError, "Can only substract Rectangle or tuple (x, y, width, height), not %s." % repr(obj)
+            raise TypeError(
+                f"Can only subtract Rectangle or tuple (x, y, width, height), not {repr(obj)}."
+            )
         x1, y1 = x + width, y + height
 
         if self:
@@ -247,27 +251,29 @@ class Rectangle(object):
                 x, y = obj
                 x1, y1 = obj
             except ValueError:
-                raise TypeError, "Should compare to Rectangle, tuple (x, y, width, height) or point (x, y), not %s." % repr(obj)
-        return x >= self.x and x1 <= self.x1 and \
-               y >= self.y and y1 <= self.y1
+                raise TypeError(
+                    f"Should compare to Rectangle, tuple (x, y, width, height) or point (x, y), not {repr(obj)}."
+                )
+        return x >= self.x and x1 <= self.x1 and y >= self.y and y1 <= self.y1
 
 
-def distance_point_point(point1, point2=(0., 0.)):
+def distance_point_point(point1, point2=(0.0, 0.0)):
     """
     Return the distance from point ``point1`` to ``point2``.
 
-    >>> '%.3f' % distance_point_point((0,0), (1,1))
+    >>> f"{distance_point_point((0,0), (1,1))}:.3f"
     '1.414'
     """
     dx = point1[0] - point2[0]
     dy = point1[1] - point2[1]
-    return sqrt(dx*dx + dy*dy)
+    return sqrt(dx * dx + dy * dy)
 
 
-def distance_point_point_fast(point1, point2):
+def distance_point_point_fast(point1, point2=(0.0, 0.0)):
     """
-    Return the distance from point ``point1`` to ``point2``. This version is
-    faster than ``distance_point_point()``, but less precise.
+    Return the distance from point ``point1`` to ``point2``. This
+    version is faster than ``distance_point_point()``, but less
+    precise.
 
     >>> distance_point_point_fast((0,0), (1,1))
     2
@@ -279,8 +285,8 @@ def distance_point_point_fast(point1, point2):
 
 def distance_rectangle_point(rect, point):
     """
-    Return the distance (fast) from a rectangle ``(x, y, width, height)`` to a
-    ``point``.
+    Return the distance (fast) from a rectangle ``(x, y, width,height)``
+    to a ``point``.
 
     >>> distance_rectangle_point(Rectangle(0, 0, 10, 10), (11, -1))
     2
@@ -308,10 +314,11 @@ def distance_rectangle_point(rect, point):
 
 def point_on_rectangle(rect, point, border=False):
     """
-    Return the point on which ``point`` can be projecten on the rectangle.
-    ``border = True`` will make sure the point is bound to the border of
-    the reactangle. Otherwise, if the point is in the rectangle, it's okay.
-    
+    Return the point on which ``point`` can be projecten on the
+    rectangle.  ``border = True`` will make sure the point is bound to
+    the border of the reactangle. Otherwise, if the point is in the
+    rectangle, it's okay.
+
     >>> point_on_rectangle(Rectangle(0, 0, 10, 10), (11, -1))
     (10, 0)
     >>> point_on_rectangle((0, 0, 10, 10), (5, 12))
@@ -357,14 +364,13 @@ def point_on_rectangle(rect, point, border=False):
 
     if x_inside and y_inside:
         # Find point on side closest to the point
-        if min(abs(rx - px), abs(rx + rw - px)) > \
-           min(abs(ry - py), abs(ry + rh - py)):
-            if py < ry + rh / 2.:
+        if min(abs(rx - px), abs(rx + rw - px)) > min(abs(ry - py), abs(ry + rh - py)):
+            if py < ry + rh / 2.0:
                 py = ry
             else:
                 py = ry + rh
         else:
-            if px < rx + rw / 2.:
+            if px < rx + rw / 2.0:
                 px = rx
             else:
                 px = rx + rw
@@ -374,8 +380,8 @@ def point_on_rectangle(rect, point, border=False):
 
 def distance_line_point(line_start, line_end, point):
     """
-    Calculate the distance of a ``point`` from a line. The line is marked
-    by begin and end point ``line_start`` and ``line_end``. 
+    Calculate the distance of a ``point`` from a line. The line is
+    marked by begin and end point ``line_start`` and ``line_end``.
 
     A tuple is returned containing the distance and point on the line.
 
@@ -386,9 +392,9 @@ def distance_line_point(line_start, line_end, point):
     >>> distance_line_point((0., 0.), (2., 4.), point=(1., 2.))
     (0.0, (1.0, 2.0))
     >>> d, p = distance_line_point((0., 0.), (2., 4.), point=(2., 2.))
-    >>> '%.3f' % d
+    >>> f"{d:.3f}"
     '0.894'
-    >>> '(%.3f, %.3f)' % p
+    >>> f"({p:.3f}, {p:.3f}"
     '(1.200, 2.400)'
     """
     # The original end point:
@@ -416,15 +422,17 @@ def distance_line_point(line_start, line_end, point):
         # Projection is on the line. multiply the line_end with the projlen
         # factor to obtain the point on the line.
         proj = line_end[0] * projlen, line_end[1] * projlen
-        return distance_point_point((proj[0] - point[0], proj[1] - point[1])),\
-               (line_start[0] + proj[0], line_start[1] + proj[1])
+        return (
+            distance_point_point((proj[0] - point[0], proj[1] - point[1])),
+            (line_start[0] + proj[0], line_start[1] + proj[1]),
+        )
 
 
 def intersect_line_line(line1_start, line1_end, line2_start, line2_end):
     """
     Find the point where the lines (segments) defined by
-    ``(line1_start, line1_end)`` and ``(line2_start, line2_end)`` intersect.
-    If no intersecion occurs, ``None`` is returned.
+    ``(line1_start, line1_end)`` and ``(line2_start, line2_end)``
+    intersect.  If no intersection occurs, ``None`` is returned.
 
     >>> intersect_line_line((3, 0), (8, 10), (0, 0), (10, 10))
     (6, 6)
@@ -443,8 +451,18 @@ def intersect_line_line(line1_start, line1_end, line2_start, line2_end):
     >>> intersect_line_line((527.5, 141.5), (336.5, 139.5), (478.0, 117.0), (478.0, 166.0))
     (478.5, 141.48167539267016)
 
-    This is a Python translation of the ``lines_intersect`` routine written
+    This is a Python translation of the ``lines_intersect``, C Code from
+    Graphics Gems II, Academic Press, Inc. The original routine was written
     by Mukesh Prasad.
+
+    EULA: The Graphics Gems code is copyright-protected. In other words, you
+    cannot claim the text of the code as your own and resell it. Using the code
+    is permitted in any program, product, or library, non-commercial or
+    commercial. Giving credit is not required, though is a nice gesture. The
+    code comes as-is, and if there are any flaws or problems with any Gems
+    code, nobody involved with Gems - authors, editors, publishers, or
+    webmasters - are to be held responsible. Basically, don't be a jerk, and
+    remember that anything free comes with no guarantee.
     """
     #
     #   This function computes whether two line segments,
@@ -473,7 +491,7 @@ def intersect_line_line(line1_start, line1_end, line2_start, line2_end):
     #        DO_INTERSECT      1
     #        COLLINEAR         2
     #
-    # Error condititions:
+    # Error conditions:
     #
     #     Depending upon the possible ranges, and particularly on 16-bit
     #     computers, care should be taken to protect from overflow.
@@ -481,33 +499,33 @@ def intersect_line_line(line1_start, line1_end, line2_start, line2_end):
     #     In the following code, 'long' values have been used for this
     #     purpose, instead of 'int'.
     #
-   
+
     x1, y1 = line1_start
     x2, y2 = line1_end
     x3, y3 = line2_start
     x4, y4 = line2_end
-    
-    #long a1, a2, b1, b2, c1, c2; /* Coefficients of line eqns. */
-    #long r1, r2, r3, r4;         /* 'Sign' values */
-    #long denom, offset, num;     /* Intermediate values */
+
+    # long a1, a2, b1, b2, c1, c2; /* Coefficients of line eqns. */
+    # long r1, r2, r3, r4;         /* 'Sign' values */
+    # long denom, offset, num;     /* Intermediate values */
 
     # Compute a1, b1, c1, where line joining points 1 and 2
     # is "a1 x  +  b1 y  +  c1  =  0".
-    
+
     a1 = y2 - y1
     b1 = x1 - x2
     c1 = x2 * y1 - x1 * y2
 
     # Compute r3 and r4.
-    
+
     r3 = a1 * x3 + b1 * y3 + c1
     r4 = a1 * x4 + b1 * y4 + c1
 
     # Check signs of r3 and r4.  If both point 3 and point 4 lie on
     # same side of line 1, the line segments do not intersect.
-    
+
     if r3 and r4 and (r3 * r4) >= 0:
-        return None # ( DONT_INTERSECT )
+        return None  # ( DONT_INTERSECT )
 
     # Compute a2, b2, c2
 
@@ -523,27 +541,28 @@ def intersect_line_line(line1_start, line1_end, line2_start, line2_end):
     # Check signs of r1 and r2.  If both point 1 and point 2 lie
     # on same side of second line segment, the line segments do
     # not intersect.
-    
-    if r1 and r2 and (r1 * r2) >= 0: #SAME_SIGNS( r1, r2 ))
-        return None # ( DONT_INTERSECT )
 
-    # Line segments intersect: compute intersection point. 
-    
-    denom = a1 * b2 - a2 * b1
-    if not denom:
-        return None # ( COLLINEAR )
-    offset = abs(denom) / 2
+    if r1 and r2 and (r1 * r2) >= 0:  # SAME_SIGNS( r1, r2 ))
+        return None  # ( DONT_INTERSECT )
 
-    # The denom/2 is to get rounding instead of truncating.  It
+    # Line segments intersect: compute intersection point.
+    # The denom / 2 is to get rounding instead of truncating.  It
     # is added or subtracted to the numerator, depending upon the
     # sign of the numerator.
-    
-    num = b1 * c2 - b2 * c1
-    x = ( (num < 0) and (num - offset) or (num + offset) ) / denom
-
-    num = a2 * c1 - a1 * c2
-    y = ( (num < 0) and (num - offset) or (num + offset) ) / denom
 
+    denom = a1 * b2 - a2 * b1
+    x_num = b1 * c2 - b2 * c1
+    y_num = a2 * c1 - a1 * c2
+    if not denom:
+        return None  # ( COLLINEAR )
+    elif isinstance(denom, float):  # denom is float, use normal division
+        offset = abs(denom) / 2
+        x = ((x_num < 0) and (x_num - offset) or (x_num + offset)) / denom
+        y = ((y_num < 0) and (y_num - offset) or (y_num + offset)) / denom
+    else:  # denom is int, use integer division
+        offset = abs(denom) // 2
+        x = ((x_num < 0) and (x_num - offset) or (x_num + offset)) // denom
+        y = ((y_num < 0) and (y_num - offset) or (y_num + offset)) // denom
     return x, y
 
 
@@ -572,8 +591,8 @@ def rectangle_intersects(recta, rectb):
 
 def rectangle_clip(recta, rectb):
     """
-    Return the clipped rectangle of ``recta`` and ``rectb``. If they do not
-    intersect, ``None`` is returned.
+    Return the clipped rectangle of ``recta`` and ``rectb``. If they
+    do not intersect, ``None`` is returned.
 
     >>> rectangle_clip((0, 0, 20, 20), (10, 10, 20, 20))
     (10, 10, 10, 10)
@@ -582,11 +601,8 @@ def rectangle_clip(recta, rectb):
     bx, by, bw, bh = rectb
     x = max(ax, bx)
     y = max(ay, by)
-    w = min(ax +aw, bx + bw) - x
-    h = min(ay +ah, by + bh) - y
+    w = min(ax + aw, bx + bw) - x
+    h = min(ay + ah, by + bh) - y
     if w < 0 or h < 0:
         return None
     return (x, y, w, h)
-
-
-# vim:sw=4:et:ai
