@@ -1,26 +1,29 @@
 """
 Basic items.
 """
-
-__version__ = "$Revision$"
-# $HeadURL$
-
+from functools import reduce
 from math import atan2
-from weakref import WeakKeyDictionary
-try:
-    # python 3.0 (better be prepared)
-    from weakref import WeakSet
-except ImportError:
-    from weakset import WeakSet
+from weakref import WeakKeyDictionary, WeakSet
 
-from matrix import Matrix
-from geometry import distance_line_point, distance_rectangle_point
-from connector import Handle, LinePort
-from solver import solvable, WEAK, NORMAL, STRONG, VERY_STRONG
-from constraint import EqualsConstraint, LessThanConstraint, LineConstraint, LineAlignConstraint
-from state import observed, reversible_method, reversible_pair, reversible_property
+from gaphas.connector import Handle, LinePort
+from gaphas.constraint import (
+    EqualsConstraint,
+    LessThanConstraint,
+    LineAlignConstraint,
+    LineConstraint,
+)
+from gaphas.geometry import distance_line_point, distance_rectangle_point
+from gaphas.matrix import Matrix
+from gaphas.solver import REQUIRED, VERY_STRONG, WEAK, solvable
+from gaphas.state import (
+    observed,
+    reversible_method,
+    reversible_pair,
+    reversible_property,
+)
 
-class Item(object):
+
+class Item:
     """
     Base class (or interface) for items on a canvas.Canvas.
 
@@ -58,7 +61,7 @@ class Item(object):
         # used by gaphas.view.GtkView to hold item 2 view matrices (view=key)
         self._matrix_i2v = WeakKeyDictionary()
         self._matrix_v2i = WeakKeyDictionary()
-	self._canvas_projections = WeakSet()
+        self._canvas_projections = WeakSet()
 
     @observed
     def _set_canvas(self, canvas):
@@ -73,11 +76,11 @@ class Item(object):
         if canvas:
             self.setup_canvas()
 
-    canvas = reversible_property(lambda s: s._canvas, _set_canvas,
-                doc="Canvas owning this item")
+    canvas = reversible_property(
+        lambda s: s._canvas, _set_canvas, doc="Canvas owning this item"
+    )
 
-    constraints = property(lambda s: s._constraints,
-                doc="Item constraints")
+    constraints = property(lambda s: s._constraints, doc="Item constraints")
 
     def setup_canvas(self):
         """
@@ -88,7 +91,6 @@ class Item(object):
         for c in self._constraints:
             add(c)
 
-
     def teardown_canvas(self):
         """
         Called when the canvas is unset for the item.
@@ -100,7 +102,6 @@ class Item(object):
         for c in self._constraints:
             remove(c)
 
-
     @observed
     def _set_matrix(self, matrix):
         """
@@ -112,12 +113,10 @@ class Item(object):
 
     matrix = reversible_property(lambda s: s._matrix, _set_matrix)
 
-
     def request_update(self, update=True, matrix=True):
         if self._canvas:
             self._canvas.request_update(self, update=update, matrix=matrix)
 
-
     def pre_update(self, context):
         """
         Perform any changes before item update here, for example:
@@ -125,13 +124,12 @@ class Item(object):
         - change matrix
         - move handles
 
-        Gaphas does not guarantee that any canvas invariant is valid at
-        this point (i.e. constraints are not solved, first handle is not in
-        position (0, 0), etc).
+        Gaphas does not guarantee that any canvas invariant is valid
+        at this point (i.e. constraints are not solved, first handle
+        is not in position (0, 0), etc).
         """
         pass
 
-
     def post_update(self, context):
         """
         Method called after item update.
@@ -139,34 +137,35 @@ class Item(object):
         If some variables should be used during drawing or in another
         update, then they should be calculated in post method.
 
-        Changing matrix or moving handles programmatically is really not
-        advised to be performed here.
+        Changing matrix or moving handles programmatically is really
+        not advised to be performed here.
 
         All canvas invariants are true.
         """
         pass
 
-
     def normalize(self):
         """
-        Update handle positions of the item, so the first handle is always
-        located at (0, 0).
-        
-        Note that, since this method basically does some housekeeping during
-        the update phase, there's no need to keep track of the changes.
+        Update handle positions of the item, so the first handle is
+        always located at (0, 0).
 
-        Alternative implementation can also be created, e.g. set (0, 0) in
-        the center of a circle or change it depending on the location of a
-        rotation point.
+        Note that, since this method basically does some housekeeping
+        during the update phase, there's no need to keep track of the
+        changes.
 
-        Returns ``True`` if some updates have been done, ``False`` otherwise.
+        Alternative implementation can also be created, e.g. set (0,
+        0) in the center of a circle or change it depending on the
+        location of a rotation point.
+
+        Returns ``True`` if some updates have been done, ``False``
+        otherwise.
 
         See ``canvas._normalize()`` for tests.
         """
         updated = False
         handles = self._handles
         if handles:
-            x, y = map(float, handles[0].pos)
+            x, y = list(map(float, handles[0].pos))
             if x:
                 self.matrix.translate(x, 0)
                 updated = True
@@ -179,7 +178,6 @@ class Item(object):
                     h.pos.y -= y
         return updated
 
-
     def draw(self, context):
         """
         Render the item to a canvas view.
@@ -187,26 +185,23 @@ class Item(object):
 
         - cairo: the Cairo Context use this one to draw
         - view: the view that is to be rendered to
-        - selected, focused, hovered, dropzone: view state of items (True/False)
-        - draw_all: a request to draw everything, for bounding box calculations
+        - selected, focused, hovered, dropzone: view state of items
+          (True/False)
         """
         pass
 
-    
     def handles(self):
         """
         Return a list of handles owned by the item.
         """
         return self._handles
 
-
     def ports(self):
         """
         Return list of ports.
         """
         return self._ports
 
-
     def point(self, pos):
         """
         Get the distance from a point (``x``, ``y``) to the item.
@@ -214,18 +209,19 @@ class Item(object):
         """
         pass
 
-
-    def constraint(self,
-            horizontal=None,
-            vertical=None,
-            left_of=None,
-            above=None,
-            line=None,
-            delta=0.0,
-            align=None):
+    def constraint(
+        self,
+        horizontal=None,
+        vertical=None,
+        left_of=None,
+        above=None,
+        line=None,
+        delta=0.0,
+        align=None,
+    ):
         """
-        Utility (factory) method to create item's internal constraint between
-        two positions or between a position and a line.
+        Utility (factory) method to create item's internal constraint
+        between two positions or between a position and a line.
 
         Position is a tuple of coordinates, i.e. ``(2, 4)``.
 
@@ -248,7 +244,7 @@ class Item(object):
          line=(p, l)
             Keep position ``p`` on line ``l``.
         """
-        cc = None # created constraint
+        cc = None  # created constraint
         if horizontal:
             p1, p2 = horizontal
             cc = EqualsConstraint(p1[1], p2[1], delta)
@@ -268,42 +264,38 @@ class Item(object):
             else:
                 cc = LineAlignConstraint(line=l, point=pos, align=align, delta=delta)
         else:
-            raise ValueError('Constraint incorrectly specified')
+            raise ValueError("Constraint incorrectly specified")
         assert cc is not None
         self._constraints.append(cc)
         return cc
 
-
     def __getstate__(self):
         """
         Persist all, but calculated values (``_matrix_?2?``).
         """
         d = dict(self.__dict__)
-        for n in ('_matrix_i2c', '_matrix_c2i', '_matrix_i2v', '_matrix_v2i'):
+        for n in ("_matrix_i2c", "_matrix_c2i", "_matrix_i2v", "_matrix_v2i"):
             try:
                 del d[n]
             except KeyError:
                 pass
-        d['_canvas_projections'] = tuple(self._canvas_projections)
+        d["_canvas_projections"] = tuple(self._canvas_projections)
         return d
 
-
     def __setstate__(self, state):
         """
         Set state. No ``__init__()`` is called.
         """
-        for n in ('_matrix_i2c', '_matrix_c2i'):
+        for n in ("_matrix_i2c", "_matrix_c2i"):
             setattr(self, n, None)
-        for n in ('_matrix_i2v', '_matrix_v2i'):
+        for n in ("_matrix_i2v", "_matrix_v2i"):
             setattr(self, n, WeakKeyDictionary())
         self.__dict__.update(state)
-        self._canvas_projections = WeakSet(state['_canvas_projections'])
+        self._canvas_projections = WeakSet(state["_canvas_projections"])
 
 
-[ NW,
-  NE,
-  SE,
-  SW ] = xrange(4)
+[NW, NE, SE, SW] = list(range(4))
+
 
 class Element(Item):
     """
@@ -314,9 +306,12 @@ class Element(Item):
      SW +---+ SE
     """
 
+    min_width = solvable(strength=REQUIRED, varname="_min_width")
+    min_height = solvable(strength=REQUIRED, varname="_min_height")
+
     def __init__(self, width=10, height=10):
-        super(Element, self).__init__()
-        self._handles = [ h(strength=VERY_STRONG) for h in [Handle]*4 ]
+        super().__init__()
+        self._handles = [h(strength=VERY_STRONG) for h in [Handle] * 4]
 
         handles = self._handles
         h_nw = handles[NW]
@@ -327,28 +322,31 @@ class Element(Item):
         # edge of element define default element ports
         self._ports = [
             LinePort(h_nw.pos, h_ne.pos),
-            LinePort(h_ne.pos, h_se.pos), 
-            LinePort(h_se.pos, h_sw.pos), 
-            LinePort(h_sw.pos, h_nw.pos)
+            LinePort(h_ne.pos, h_se.pos),
+            LinePort(h_se.pos, h_sw.pos),
+            LinePort(h_sw.pos, h_nw.pos),
         ]
 
-        # setup constraints
+        # initialize min_x variables
+        self.min_width, self.min_height = 10, 10
+
         self.constraint(horizontal=(h_nw.pos, h_ne.pos))
-        self.constraint(horizontal=(h_se.pos, h_sw.pos))
+        self.constraint(horizontal=(h_sw.pos, h_se.pos))
         self.constraint(vertical=(h_nw.pos, h_sw.pos))
-        self.constraint(vertical=(h_se.pos, h_ne.pos))
+        self.constraint(vertical=(h_ne.pos, h_se.pos))
 
         # create minimal size constraints
-        self._c_min_w = self.constraint(left_of=(h_nw.pos, h_se.pos), delta=10)
-        self._c_min_h = self.constraint(above=(h_nw.pos, h_se.pos), delta=10)
+        self.constraint(left_of=(h_nw.pos, h_se.pos), delta=self._min_width)
+        self.constraint(above=(h_nw.pos, h_se.pos), delta=self._min_height)
 
-        # set width/height when minimal size constraints exist
         self.width = width
         self.height = height
 
+        # TODO: constraints that calculate width and height based on handle pos
+        # self.constraints.append(EqualsConstraint(p1[1], p2[1], delta))
 
     def setup_canvas(self):
-        super(Element, self).setup_canvas()
+        super().setup_canvas()
 
         # Trigger solver to honour width/height by SE handle pos
         self._handles[SE].pos.x.dirty()
@@ -365,12 +363,9 @@ class Element(Item):
         >>> b._handles[SE].pos.x
         Variable(20, 40)
         """
-        if width < self.min_width:
-            width = self.min_width
         h = self._handles
         h[SE].pos.x = h[NW].pos.x + width
 
-
     def _get_width(self):
         """
         Width of the box, calculated as the distance from the left and
@@ -389,14 +384,12 @@ class Element(Item):
         20.0
         >>> b.height = 2
         >>> b.height
-        10.0
+        2.0
         >>> b._handles[NW].pos.y
         Variable(0, 40)
         >>> b._handles[SE].pos.y
-        Variable(10, 40)
+        Variable(2, 40)
         """
-        if height < self.min_height:
-            height = self.min_height
         h = self._handles
         h[SE].pos.y = h[NW].pos.y + height
 
@@ -409,35 +402,26 @@ class Element(Item):
 
     height = property(_get_height, _set_height)
 
-    @observed
-    def _set_min_width(self, min_width):
+    def normalize(self):
         """
-        Set minimal width.
+        Normalize only NW and SE handles
         """
-        if min_width < 0:
-            raise ValueError, 'Minimal width cannot be less than 0'
+        updated = False
+        handles = self._handles
+        handles = (handles[NW], handles[SE])
+        x, y = list(map(float, handles[0].pos))
+        if x:
+            self.matrix.translate(x, 0)
+            updated = True
+            for h in handles:
+                h.pos.x -= x
+        if y:
+            self.matrix.translate(0, y)
+            updated = True
+            for h in handles:
+                h.pos.y -= y
+        return updated
 
-        self._c_min_w.delta = min_width
-        if self.canvas:
-            self.canvas.solver.request_resolve_constraint(self._c_min_w)
-
-    min_width = reversible_property(lambda s: s._c_min_w.delta, _set_min_width)
-
-    @observed
-    def _set_min_height(self, min_height):
-        """
-        Set minimal height.
-        """
-        if min_height < 0:
-            raise ValueError, 'Minimal height cannot be less than 0'
-
-        self._c_min_h.delta = min_height
-        if self.canvas:
-            self.canvas.solver.request_resolve_constraint(self._c_min_h)
-
-    min_height = reversible_property(lambda s: s._c_min_h.delta, _set_min_height)
-
-        
     def point(self, pos):
         """
         Distance from the point (x, y) to the item.
@@ -448,7 +432,9 @@ class Element(Item):
         """
         h = self._handles
         pnw, pse = h[NW].pos, h[SE].pos
-        return distance_rectangle_point(map(float, (pnw.x, pnw.y, pse.x, pse.y)), pos)
+        return distance_rectangle_point(
+            list(map(float, (pnw.x, pnw.y, pse.x, pse.y))), pos
+        )
 
 
 class Line(Item):
@@ -456,22 +442,24 @@ class Line(Item):
     A Line item.
 
     Properties:
-     - fuzziness (0.0..n): an extra margin that should be taken into account
-         when calculating the distance from the line (using point()).
-     - orthogonal (bool): wherther or not the line should be orthogonal
-         (only straight angles)
+     - fuzziness (0.0..n): an extra margin that should be taken into
+         account when calculating the distance from the line (using
+         point()).
+     - orthogonal (bool): whether or not the line should be
+         orthogonal (only straight angles)
      - horizontal: first line segment is horizontal
      - line_width: width of the line to be drawn
 
-    This line also supports arrow heads on both the begin and end of the
-    line. These are drawn with the methods draw_head(context) and
-    draw_tail(context). The coordinate system is altered so the methods do
-    not have to know about the angle of the line segment (e.g. drawing a line
-    from (10, 10) via (0, 0) to (10, -10) will draw an arrow point).
+    This line also supports arrow heads on both the begin and end of
+    the line. These are drawn with the methods draw_head(context) and
+    draw_tail(context). The coordinate system is altered so the
+    methods do not have to know about the angle of the line segment
+    (e.g. drawing a line from (10, 10) via (0, 0) to (10, -10) will
+    draw an arrow point).
     """
 
     def __init__(self):
-        super(Line, self).__init__()
+        super().__init__()
         self._handles = [Handle(connectable=True), Handle((10, 10), connectable=True)]
         self._ports = []
         self._update_ports()
@@ -501,7 +489,7 @@ class Line(Item):
         observed, so the undo system will update the contents properly
         """
         if not self.canvas:
-            self._orthogonal_constraints = orthogonal and [ None ] or []
+            self._orthogonal_constraints = orthogonal and [None] or []
             return
 
         for c in self._orthogonal_constraints:
@@ -512,16 +500,16 @@ class Line(Item):
             return
 
         h = self._handles
-        #if len(h) < 3:
+        # if len(h) < 3:
         #    self.split_segment(0)
-        eq = EqualsConstraint #lambda a, b: a - b
+        eq = EqualsConstraint  # lambda a, b: a - b
         add = self.canvas.solver.add_constraint
         cons = []
         rest = self._horizontal and 1 or 0
         for pos, (h0, h1) in enumerate(zip(h, h[1:])):
             p0 = h0.pos
             p1 = h1.pos
-            if pos % 2 == rest: # odd
+            if pos % 2 == rest:  # odd
                 cons.append(add(eq(a=p0.x, b=p1.x)))
             else:
                 cons.append(add(eq(a=p0.y, b=p1.y)))
@@ -533,11 +521,14 @@ class Line(Item):
     @observed
     def _set_orthogonal_constraints(self, orthogonal_constraints):
         """
-        Setter for the constraints maintained. Required for the undo system.
+        Setter for the constraints maintained. Required for the undo
+        system.
         """
         self._orthogonal_constraints = orthogonal_constraints
 
-    reversible_property(lambda s: s._orthogonal_constraints, _set_orthogonal_constraints)
+    reversible_property(
+        lambda s: s._orthogonal_constraints, _set_orthogonal_constraints
+    )
 
     @observed
     def _set_orthogonal(self, orthogonal):
@@ -547,17 +538,22 @@ class Line(Item):
         False
         """
         if orthogonal and len(self.handles()) < 3:
-            raise ValueError, "Can't set orthogonal line with less than 3 handles"
+            raise ValueError("Can't set orthogonal line with less than 3 handles")
         self._update_orthogonal_constraints(orthogonal)
 
-    orthogonal = reversible_property(lambda s: bool(s._orthogonal_constraints), _set_orthogonal)
+    orthogonal = reversible_property(
+        lambda s: bool(s._orthogonal_constraints), _set_orthogonal
+    )
 
     @observed
     def _inner_set_horizontal(self, horizontal):
         self._horizontal = horizontal
 
-    reversible_method(_inner_set_horizontal, _inner_set_horizontal,
-                      {'horizontal': lambda horizontal: not horizontal })
+    reversible_method(
+        _inner_set_horizontal,
+        _inner_set_horizontal,
+        {"horizontal": lambda horizontal: not horizontal},
+    )
 
     def _set_horizontal(self, horizontal):
         """
@@ -577,14 +573,14 @@ class Line(Item):
         """
         Setup constraints. In this case orthogonal.
         """
-        super(Line, self).setup_canvas()
+        super().setup_canvas()
         self._update_orthogonal_constraints(self.orthogonal)
 
     def teardown_canvas(self):
         """
         Remove constraints created in setup_canvas().
         """
-        super(Line, self).teardown_canvas()
+        super().teardown_canvas()
         for c in self._orthogonal_constraints:
             self.canvas.solver.remove_constraint(c)
 
@@ -596,8 +592,11 @@ class Line(Item):
     def _reversible_remove_handle(self, handle):
         self._handles.remove(handle)
 
-    reversible_pair(_reversible_insert_handle, _reversible_remove_handle, \
-            bind1={'index': lambda self, handle: self._handles.index(handle)})
+    reversible_pair(
+        _reversible_insert_handle,
+        _reversible_remove_handle,
+        bind1={"index": lambda self, handle: self._handles.index(handle)},
+    )
 
     @observed
     def _reversible_insert_port(self, index, port):
@@ -607,29 +606,29 @@ class Line(Item):
     def _reversible_remove_port(self, port):
         self._ports.remove(port)
 
-    reversible_pair(_reversible_insert_port, _reversible_remove_port, \
-            bind1={'index': lambda self, port: self._ports.index(port)})
-
+    reversible_pair(
+        _reversible_insert_port,
+        _reversible_remove_port,
+        bind1={"index": lambda self, port: self._ports.index(port)},
+    )
 
     def _create_handle(self, pos, strength=WEAK):
         return Handle(pos, strength=strength)
 
-    
     def _create_port(self, p1, p2):
         return LinePort(p1, p2)
 
-
     def _update_ports(self):
         """
-        Update line ports.
+        Update line ports. This destroys all previously created ports
+        and should only be used when initializing the line.
         """
-        assert len(self._handles) >= 2, 'Not enough segments'
+        assert len(self._handles) >= 2, "Not enough segments"
         self._ports = []
         handles = self._handles
         for h1, h2 in zip(handles[:-1], handles[1:]):
             self._ports.append(self._create_port(h1.pos, h2.pos))
 
-
     def opposite(self, handle):
         """
         Given the handle of one end of the line, return the other end.
@@ -640,12 +639,12 @@ class Line(Item):
         elif handle is handles[-1]:
             return handles[0]
         else:
-            raise KeyError('Handle is not an end handle')
+            raise KeyError("Handle is not an end handle")
 
     def post_update(self, context):
         """
         """
-        super(Line, self).post_update(context)
+        super().post_update(context)
         h0, h1 = self._handles[:2]
         p0, p1 = h0.pos, h1.pos
         self._head_angle = atan2(p1.y - p0.y, p1.x - p0.x)
@@ -653,25 +652,6 @@ class Line(Item):
         p1, p0 = h1.pos, h0.pos
         self._tail_angle = atan2(p1.y - p0.y, p1.x - p0.x)
 
-    def closest_segment(self, pos):
-        """
-        Obtain a tuple (distance, point_on_line, segment).
-        Distance is the distance from point to the closest line segment 
-        Point_on_line is the reflection of the point on the line.
-        Segment is the line segment closest to (x, y)
-
-        >>> a = Line()
-        >>> a.closest_segment((4, 5))
-        (0.70710678118654757, (4.5, 4.5), 0)
-        """
-        h = self._handles
-        hpos = map(getattr, h, ['pos'] * len(h))
-
-        # create a list of (distance, point_on_line) tuples:
-        distances = map(distance_line_point, hpos[:-1], hpos[1:], [pos] * (len(hpos) - 1))
-        distances, pols = zip(*distances)
-        return reduce(min, zip(distances, pols, range(len(distances))))
-
     def point(self, pos):
         """
         >>> a = Line()
@@ -679,12 +659,16 @@ class Line(Item):
         >>> a._handles.append(a._create_handle((30, 30)))
         >>> a.point((-1, 0))
         1.0
-        >>> '%.3f' % a.point((5, 4))
+        >>> f"{a.point((5, 4)):.3f}
         '2.942'
-        >>> '%.3f' % a.point((29, 29))
+        >>> f"{a.point((29, 29)):.3f}
         '0.784'
         """
-        distance, point, segment = self.closest_segment(pos)
+        hpos = [h.pos for h in self._handles]
+
+        distance, _point = min(
+            map(distance_line_point, hpos[:-1], hpos[1:], [pos] * (len(hpos) - 1))
+        )
         return max(0, distance - self.fuzziness)
 
     def draw_head(self, context):
@@ -699,12 +683,12 @@ class Line(Item):
         """
         context.cairo.line_to(0, 0)
 
-
     def draw(self, context):
         """
         Draw the line itself.
         See Item.draw(context).
         """
+
         def draw_line_end(pos, angle, draw):
             cr = context.cairo
             cr.save()
@@ -723,19 +707,5 @@ class Line(Item):
         draw_line_end(self._handles[-1].pos, self._tail_angle, self.draw_tail)
         cr.stroke()
 
-        ### debug code to draw line ports
-        ### cr.set_line_width(1)
-        ### cr.set_source_rgb(1.0, 0.0, 0.0)
-        ### for p in self.ports():
-        ###     cr.move_to(*p.start)
-        ###     cr.line_to(*p.end)
-        ### cr.stroke()
 
-
-
-__test__ = {
-    'Line._set_orthogonal': Line._set_orthogonal,
-    }
-
-
-# vim: sw=4:et:ai
+__test__ = {"Line._set_orthogonal": Line._set_orthogonal}
