@@ -2,13 +2,13 @@
 Quadtree
 ========
 
-A quadtree is a tree data structure in which each internal node has up to four
-children. Quadtrees are most often used to partition a two dimensional space by
-recursively subdividing it into four quadrants or regions. The regions may be
-square or rectangular, or may have arbitrary shapes. This data structure was
-named a quadtree by Raphael Finkel and J.L. Bentley in 1974. A similar
-partitioning is also known as a Q-tree. All forms of Quadtrees share some
-common features:
+A quadtree is a tree data structure in which each internal node has up
+to four children. Quadtrees are most often used to partition a two
+dimensional space by recursively subdividing it into four quadrants or
+regions. The regions may be square or rectangular, or may have
+arbitrary shapes. This data structure was named a quadtree by Raphael
+Finkel and J.L. Bentley in 1974. A similar partitioning is also known
+as a Q-tree. All forms of Quadtrees share some common features:
 
 * They decompose space into adaptable cells.
 * Each cell (or bucket) has a maximum capacity.
@@ -17,15 +17,12 @@ common features:
 
 (From Wikipedia, the free encyclopedia)
 """
-
-__version__ = "$Revision$"
-# $HeadURL$
-
 import operator
-from geometry import rectangle_contains, rectangle_intersects, rectangle_clip
+
+from gaphas.geometry import rectangle_clip, rectangle_contains, rectangle_intersects
 
 
-class Quadtree(object):
+class Quadtree:
     """
     The Quad-tree.
 
@@ -33,7 +30,7 @@ class Quadtree(object):
 
     >>> qtree = Quadtree((0, 0, 100, 100))
     >>> for i in range(20):
-    ...     qtree.add('%d' % i, ((i * 4) % 90, (i * 10) % 90, 10, 10))
+    ...     qtree.add(f"{i}", ((i * 4) % 90, (i * 10) % 90, 10, 10))
     >>> len(qtree)
     20
     >>> qtree.dump() # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
@@ -43,11 +40,11 @@ class Quadtree(object):
        <....QuadtreeBucket object at 0x...> (0, 0, 50.0, 50.0)
          0 (0, 0, 10, 10)
          1 (4, 10, 10, 10)
-         10 (40, 10, 10, 10)
          2 (8, 20, 10, 10)
          3 (12, 30, 10, 10)
          4 (16, 40, 10, 10)
          9 (36, 0, 10, 10)
+         10 (40, 10, 10, 10)
        <....QuadtreeBucket object at 0x...> (50.0, 0, 50.0, 50.0)
          13 (52, 40, 10, 10)
          18 (72, 0, 10, 10)
@@ -85,22 +82,20 @@ class Quadtree(object):
     def __init__(self, bounds=(0, 0, 0, 0), capacity=10):
         """
         Create a new Quadtree instance.
-        
-        Bounds is the boundries of the quadtree. this is fixed and do not
+
+        Bounds is the boundaries of the quadtree. this is fixed and do not
         change depending on the contents.
-        
+
         Capacity defines the number of elements in one tree bucket (default: 10)
         """
         self._capacity = capacity
         self._bucket = QuadtreeBucket(bounds, capacity)
 
         # Easy lookup item->(bounds, data, clipped bounds) mapping
-        self._ids = dict()
-
+        self._ids = {}
 
     bounds = property(lambda s: s._bucket.bounds)
 
-
     def resize(self, bounds):
         """
         Resize the tree.
@@ -109,11 +104,10 @@ class Quadtree(object):
         self._bucket = QuadtreeBucket(bounds, self._capacity)
         self.rebuild()
 
-
     def get_soft_bounds(self):
         """
-        Calculate the size of all items in the tree. This size may be beyond
-        the limits of the tree itself.
+        Calculate the size of all items in the tree. This size may be
+        beyond the limits of the tree itself.
 
         Returns a tuple (x, y, width, height).
 
@@ -130,24 +124,33 @@ class Quadtree(object):
         >>> qtree.bounds
         (0, 0, 0, 0)
         """
-        x_y_w_h = zip(*map(operator.getitem, self._ids.itervalues(), [0] * len(self._ids)))
+        x_y_w_h = list(
+            zip(
+                *list(
+                    map(
+                        operator.getitem,
+                        iter(list(self._ids.values())),
+                        [0] * len(self._ids),
+                    )
+                )
+            )
+        )
         if not x_y_w_h:
             return 0, 0, 0, 0
         x0 = min(x_y_w_h[0])
         y0 = min(x_y_w_h[1])
         add = operator.add
-        x1 = max(map(add, x_y_w_h[0], x_y_w_h[2]))
-        y1 = max(map(add, x_y_w_h[1], x_y_w_h[3]))
+        x1 = max(list(map(add, x_y_w_h[0], x_y_w_h[2])))
+        y1 = max(list(map(add, x_y_w_h[1], x_y_w_h[3])))
         return (x0, y0, x1 - x0, y1 - y0)
 
     soft_bounds = property(get_soft_bounds)
 
-
     def add(self, item, bounds, data=None):
         """
         Add an item to the tree.
-        If an item already exists, its bounds are updated and the item is
-        moved to the right bucket.
+        If an item already exists, its bounds are updated and the item
+        is moved to the right bucket.
         Data can be used to add some extra info to the item
         """
         # Clip item bounds to fit in top-level bucket
@@ -161,8 +164,11 @@ class Quadtree(object):
                 assert item in bucket.items
                 # Fast lane, if item moved just a little it may still reside
                 # in the same bucket. We do not need to search from top-level.
-                if bucket and clipped_bounds and \
-                        rectangle_contains(clipped_bounds, bucket.bounds):
+                if (
+                    bucket
+                    and clipped_bounds
+                    and rectangle_contains(clipped_bounds, bucket.bounds)
+                ):
                     bucket.update(item, clipped_bounds)
                     self._ids[item] = (bounds, data, clipped_bounds)
                     return
@@ -173,7 +179,6 @@ class Quadtree(object):
             self._bucket.find_bucket(clipped_bounds).add(item, clipped_bounds)
         self._ids[item] = (bounds, data, clipped_bounds)
 
-
     def remove(self, item):
         """
         Remove an item from the tree.
@@ -183,7 +188,6 @@ class Quadtree(object):
         if clipped_bounds:
             self._bucket.find_bucket(clipped_bounds).remove(item)
 
-
     def clear(self):
         """
         Remove all items from the tree.
@@ -191,7 +195,6 @@ class Quadtree(object):
         self._bucket.clear()
         self._ids.clear()
 
-
     def rebuild(self):
         """
         Rebuild the tree structure.
@@ -199,43 +202,38 @@ class Quadtree(object):
         # Clean bucket and items:
         self._bucket.clear()
 
-        for item, (bounds, data, _) in dict(self._ids).iteritems():
+        for item, (bounds, data, _) in list(dict(self._ids).items()):
             clipped_bounds = rectangle_clip(bounds, self._bucket.bounds)
             if clipped_bounds:
                 self._bucket.find_bucket(clipped_bounds).add(item, clipped_bounds)
             self._ids[item] = (bounds, data, clipped_bounds)
 
-
     def get_bounds(self, item):
         """
         Return the bounding box for the given item.
         """
         return self._ids[item][0]
 
-
     def get_data(self, item):
         """
         Return the data for the given item, None if no data was provided.
         """
         return self._ids[item][1]
 
-
     def get_clipped_bounds(self, item):
         """
-        Return the bounding box for the given item. The bounding box is clipped
-        on the boundries of the tree (provided on construction or with
-        resize()).
+        Return the bounding box for the given item. The bounding box
+        is clipped on the boundaries of the tree (provided on
+        construction or with resize()).
         """
         return self._ids[item][2]
 
-
     def find_inside(self, rect):
         """
         Find all items in the given rectangle (x, y, with, height).
         Returns a set.
         """
         return set(self._bucket.find(rect, method=rectangle_contains))
-        
 
     def find_intersect(self, rect):
         """
@@ -244,7 +242,6 @@ class Quadtree(object):
         Returns a set.
         """
         return set(self._bucket.find(rect, method=rectangle_intersects))
-        
 
     def __len__(self):
         """
@@ -252,14 +249,12 @@ class Quadtree(object):
         """
         return len(self._ids)
 
-
     def __contains__(self, item):
         """
         Check if an item is in tree.
         """
         return item in self._ids
 
-
     def dump(self):
         """
         Print structure to stdout.
@@ -267,7 +262,7 @@ class Quadtree(object):
         self._bucket.dump()
 
 
-class QuadtreeBucket(object):
+class QuadtreeBucket:
     """
     A node in a Quadtree structure.
     """
@@ -282,25 +277,26 @@ class QuadtreeBucket(object):
         self.items = {}
         self._buckets = []
 
-
     def add(self, item, bounds):
         """
         Add an item to the quadtree.
-        The bucket is split when nessecary.
+        The bucket is split when necessary.
         Items are otherwise added to this bucket, not some sub-bucket.
         """
         assert rectangle_contains(bounds, self.bounds)
         # create new subnodes if threshold is reached
         if not self._buckets and len(self.items) >= self.capacity:
             x, y, w, h = self.bounds
-            rw, rh = w / 2., h / 2.
+            rw, rh = w / 2.0, h / 2.0
             cx, cy = x + rw, y + rh
-            self._buckets = [QuadtreeBucket((x, y, rw, rh), self.capacity),
-                             QuadtreeBucket((cx, y, rw, rh), self.capacity),
-                             QuadtreeBucket((x, cy, rw, rh), self.capacity),
-                             QuadtreeBucket((cx, cy, rw, rh), self.capacity)]
+            self._buckets = [
+                QuadtreeBucket((x, y, rw, rh), self.capacity),
+                QuadtreeBucket((cx, y, rw, rh), self.capacity),
+                QuadtreeBucket((x, cy, rw, rh), self.capacity),
+                QuadtreeBucket((cx, cy, rw, rh), self.capacity),
+            ]
             # Add items to subnodes
-            items = self.items.items()
+            items = list(self.items.items())
             self.items.clear()
             for i, b in items:
                 self.find_bucket(b).add(i, b)
@@ -308,14 +304,12 @@ class QuadtreeBucket(object):
         else:
             self.items[item] = bounds
 
-
     def remove(self, item):
         """
         Remove an item from the quadtree bucket.
         The item should be contained by *this* bucket (not a sub-bucket).
         """
         del self.items[item]
-        
 
     def update(self, item, new_bounds):
         """
@@ -327,31 +321,29 @@ class QuadtreeBucket(object):
         self.remove(item)
         self.find_bucket(new_bounds).add(item, new_bounds)
 
-
     def find_bucket(self, bounds):
         """
         Find the bucket that holds a bounding box.
 
-        This method should be used to find a bucket that fits, before add()
-        or remove() is called.
+        This method should be used to find a bucket that fits, before
+        add() or remove() is called.
         """
-        if self._buckets:
-            sx, sy, sw, sh = self.bounds
-            cx, cy = sx + sw / 2., sy + sh / 2.
-            x, y, w, h = bounds
-            index = 0
-            if x >= cx:
-                index += 1
-            elif x + w > cx:
-                return self
-
-            if y >= cy:
-                index += 2
-            elif y + h > cy:
-                return self
-            return self._buckets[index].find_bucket(bounds)
-        return self
+        if not self._buckets:
+            return self
+        sx, sy, sw, sh = self.bounds
+        cx, cy = sx + sw / 2.0, sy + sh / 2.0
+        x, y, w, h = bounds
+        index = 0
+        if x >= cx:
+            index += 1
+        elif x + w > cx:
+            return self
 
+        if y >= cy:
+            index += 2
+        elif y + h > cy:
+            return self
+        return self._buckets[index].find_bucket(bounds)
 
     def find(self, rect, method):
         """
@@ -361,13 +353,11 @@ class QuadtreeBucket(object):
         Returns an iterator.
         """
         if rectangle_intersects(rect, self.bounds):
-            for item, bounds in self.items.iteritems():
+            for item, bounds in list(self.items.items()):
                 if method(bounds, rect):
                     yield item
             for bucket in self._buckets:
-                for item in bucket.find(rect, method=method):
-                    yield item
-                
+                yield from bucket.find(rect, method=method)
 
     def clear(self):
         """
@@ -376,14 +366,10 @@ class QuadtreeBucket(object):
         del self._buckets[:]
         self.items.clear()
 
-
-    def dump(self, indent=''):
-       print indent, self, self.bounds
-       indent += '   '
-       for item, bounds in sorted(self.items.iteritems()):
-           print indent, item, bounds
-       for bucket in self._buckets:
-           bucket.dump(indent)
-
-
-# vim:sw=4:et:ai
+    def dump(self, indent=""):
+        print(indent, self, self.bounds)
+        indent += "   "
+        for item, bounds in sorted(self.items.items(), key=lambda items: items[1]):
+            print(indent, item, bounds)
+        for bucket in self._buckets:
+            bucket.dump(indent)
