@@ -1,21 +1,23 @@
 """
-Constraint solver allows to define constraint between two or more different
-variables and keep this constraint always true when one or more of the
-constrained variables change. For example, one may want to keep two
-variables always equal.
+Constraint solver allows to define constraint between two or more
+different variables and keep this constraint always true when one or
+more of the constrained variables change. For example, one may want to
+keep two variables always equal.
 
-Variables change and at some point of time we want to make all constraints
-valid again. This process is called solving constraints.
+Variables change and at some point of time we want to make all
+constraints valid again. This process is called solving constraints.
 
-Gaphas' solver allows to define constraints between Variable instances.
-Constraint classes are defined in `gaphas.constraint` module.
+Gaphas' solver allows to define constraints between Variable
+instances.  Constraint classes are defined in `gaphas.constraint`
+module.
 
 How It Works
 ------------
-Every constraint contains list of variables and has to be registered in
-solver object. Variables change (`Variable.dirty()`, `Solver.request_resolve()`
-methods) and their constraints are marked by solver as dirty. To solve
-constraints, solver loops through dirty constraints and asks constraint for
+Every constraint contains list of variables and has to be registered
+in solver object. Variables change (`Variable.dirty()`,
+`Solver.request_resolve()` methods) and their constraints are marked
+by solver as dirty. To solve constraints, solver loops through dirty
+constraints and asks constraint for
 a variable (called weakest variable), which
 
 - has the lowest strength
@@ -26,18 +28,14 @@ a variable (called weakest variable), which
 
 (weakest variable invariants defined above)
 
-Having weakest variable (`constraint.Constraint.weakest()` method) every
-constraint is being asked to solve itself (`constraint.Constraint.solve_for()`
-method) changing appropriate variables to make the constraint valid again.
+Having weakest variable (`constraint.Constraint.weakest()` method)
+every constraint is being asked to solve itself
+(`constraint.Constraint.solve_for()` method) changing appropriate
+variables to make the constraint valid again.
 """
+from gaphas.state import observed, reversible_pair, reversible_property
+from gaphas.projections import Projection
 
-from __future__ import division
-
-__version__ = "$Revision$"
-# $HeadURL$
-
-from operator import isCallable
-from state import observed, reversible_pair, reversible_property
 
 # epsilon for float comparison
 # is simple abs(x - y) > EPSILON enough for canvas needs?
@@ -52,14 +50,15 @@ VERY_STRONG = 40
 REQUIRED = 100
 
 
-class Variable(object):
-    """
-    Representation of a variable in the constraint solver.
-    Each Variable has a @value and a @strength. Ina constraint the
-    weakest variables are changed.
-    
-    You can even do some calculating with it. The Variable always represents
-    a float variable.
+class Variable:
+    """Representation of a variable in the constraint solver.
+
+    Each Variable has a @value and a @strength. In a constraint the weakest
+    variables are changed.
+
+    You can even do some calculating with it. The Variable always represents a
+    float variable.
+
     """
 
     def __init__(self, value=0.0, strength=NORMAL):
@@ -70,6 +69,9 @@ class Variable(object):
         self._solver = None
         self._constraints = set()
 
+    def __hash__(self):
+        return object.__hash__(self)
+
     @observed
     def _set_strength(self, strength):
         self._strength = strength
@@ -80,9 +82,9 @@ class Variable(object):
 
     def dirty(self):
         """
-        Mark the variable dirty in both the constraint solver and attached
-        constraints.
-        
+        Mark the variable dirty in both the constraint solver and
+        attached constraints.
+
         Variables are marked dirty also during constraints solving to
         solve all dependent constraints, i.e. two equals constraints
         between 3 variables.
@@ -97,14 +99,14 @@ class Variable(object):
     def set_value(self, value):
         oldval = self._value
         if abs(oldval - value) > EPSILON:
-            #print id(self), oldval, value
             self._value = float(value)
             self.dirty()
 
     value = reversible_property(lambda s: s._value, set_value)
 
     def __str__(self):
-        return 'Variable(%g, %d)' % (self._value, self._strength)
+        return f"Variable({self._value:g}, {self._strength:d})"
+
     __repr__ = __str__
 
     def __float__(self):
@@ -309,62 +311,7 @@ class Variable(object):
         return self._value.__rtruediv__(other)
 
 
-class Projection(object):
-    """
-    Projections are used to convert values from one space to another,
-    e.g. from Canvas to Item space or visa versa.
-
-    In order to be a Projection the ``value`` and ``strength`` properties
-    should be implemented and a method named ``variable()`` should be present.
-
-    Projections should inherit from this class.
-
-    Projections may be nested.
-
-    This default implementation projects a variable to it's own:
-
-    >>> v = Variable(4.0)
-    >>> v
-    Variable(4, 20)
-    >>> p = Projection(v)
-    >>> p.value
-    4.0
-    >>> p.value = -1
-    >>> p.value
-    -1.0
-    >>> v.value
-    -1.0
-    >>> p.strength
-    20
-    >>> p.variable()
-    Variable(-1, 20)
-    """
-
-    def __init__(self, var):
-        self._var = var
-
-    def _set_value(self, value):
-        self._var.value = value
-
-    value = property(lambda s: s._var.value, _set_value)
-
-    strength = property(lambda s: s._var.strength)
-
-    def variable(self):
-        """
-        Return the variable owned by the projection.
-        """
-        return self._var
-
-    def __float__(self):
-        return float(self.variable()._value)
-
-    def __str__(self):
-        return '%s(%s)' % (self.__class__.__name__, self.variable())
-    __repr__ = __str__
-
-
-class Solver(object):
+class Solver:
     """
     Solve constraints. A constraint should have accompanying
     variables.
@@ -378,19 +325,18 @@ class Solver(object):
 
     constraints = property(lambda s: s._constraints)
 
-
     def request_resolve(self, variable, projections_only=False):
         """
-        Mark a variable as "dirty". This means it it solved the next time
-        the constraints are resolved.
+        Mark a variable as "dirty". This means it it solved the next
+        time the constraints are resolved.
 
         If projections_only is set to True, only constraints using the
-        variable through a Projection instance (e.i. variable itself is not
-        in `constraint.Constraint.variables()`) are marked.
+        variable through a Projection instance (e.i. variable itself
+        is not in `constraint.Constraint.variables()`) are marked.
 
         Example:
 
-        >>> from constraint import EquationConstraint
+        >>> from gaphas.constraint import EquationConstraint
         >>> a, b, c = Variable(1.0), Variable(2.0), Variable(3.0)
         >>> s = Solver()
         >>> c_eq = EquationConstraint(lambda a,b: a+b, a=a, b=b)
@@ -424,19 +370,20 @@ class Solver(object):
                     c.mark_dirty(variable)
                     self._marked_cons.append(c)
                     if self._marked_cons.count(c) > 100:
-                        raise JuggleError, 'Variable juggling detected, constraint %s resolved %d times out of %d' % (c, self._marked_cons.count(c), len(self._marked_cons))
-
+                        raise JuggleError(
+                            f"Variable juggling detected, constraint {c} resolved {self._marked_cons.count(c)} times out of {len(self._marked_cons)}"
+                        )
 
     @observed
     def add_constraint(self, constraint):
         """
         Add a constraint.
-        The actual constraint is returned, so the constraint can be removed
-        later on.
+        The actual constraint is returned, so the constraint can be
+        removed later on.
 
         Example:
 
-        >>> from constraint import EquationConstraint
+        >>> from gaphas.constraint import EquationConstraint
         >>> s = Solver()
         >>> a, b = Variable(), Variable(2.0)
         >>> s.add_constraint(EquationConstraint(lambda a, b: a -b, a=a, b=b))
@@ -450,7 +397,7 @@ class Solver(object):
         >>> len(s._constraints)
         1
         """
-        assert constraint, 'No constraint (%s)' % (constraint,)
+        assert constraint, f"No constraint ({constraint})"
         self._constraints.add(constraint)
         self._marked_cons.append(constraint)
         constraint._solver_has_projections = False
@@ -460,7 +407,6 @@ class Solver(object):
                 constraint._solver_has_projections = True
             v._constraints.add(constraint)
             v._solver = self
-        #print 'added constraint', constraint
         return constraint
 
     @observed
@@ -468,7 +414,7 @@ class Solver(object):
         """
         Remove a constraint from the solver
 
-        >>> from constraint import EquationConstraint
+        >>> from gaphas.constraint import EquationConstraint
         >>> s = Solver()
         >>> a, b = Variable(), Variable(2.0)
         >>> c = s.add_constraint(EquationConstraint(lambda a, b: a -b, a=a, b=b))
@@ -478,13 +424,13 @@ class Solver(object):
         >>> s._marked_cons
         []
         >>> s._constraints
-        set([])
+        set()
 
         Removing a constraint twice has no effect:
 
         >>> s.remove_constraint(c)
         """
-        assert constraint, 'No constraint (%s)' % (constraint,)
+        assert constraint, f"No constraint ({constraint})"
         for v in constraint.variables():
             while isinstance(v, Projection):
                 v = v.variable()
@@ -495,21 +441,19 @@ class Solver(object):
 
     reversible_pair(add_constraint, remove_constraint)
 
-
     def request_resolve_constraint(self, c):
         """
         Request resolving a constraint.
         """
         self._marked_cons.append(c)
 
-
     def constraints_with_variable(self, *variables):
         """
         Return an iterator of constraints that work with variable.
         The variable in question should be exposed by the constraints
         `constraint.Constraint.variables()` method.
 
-        >>> from constraint import EquationConstraint
+        >>> from gaphas.constraint import EquationConstraint
         >>> s = Solver()
         >>> a, b, c = Variable(), Variable(2.0), Variable(4.0)
         >>> eq_a_b = s.add_constraint(EquationConstraint(lambda a, b: a -b, a=a, b=b))
@@ -567,18 +511,17 @@ class Solver(object):
                     else:
                         found = False
                     if not found:
-                        break # quit for loop, variable not in constraint
+                        break  # quit for loop, variable not in constraint
                 else:
-                    # All iteration have completed succesfully,
+                    # All iteration have completed successfully,
                     # so all variables are in the constraint
                     yield c
-                    
 
     def solve(self):
         """
         Example:
 
-        >>> from constraint import EquationConstraint
+        >>> from gaphas.constraint import EquationConstraint
         >>> a, b, c = Variable(1.0), Variable(2.0), Variable(3.0)
         >>> s = Solver()
         >>> s.add_constraint(EquationConstraint(lambda a,b: a+b, a=a, b=b))
@@ -616,8 +559,7 @@ class Solver(object):
             while n < len(marked_cons):
                 c = marked_cons[n]
                 if not c.disabled:
-                    wvar = c.weakest()
-                    c.solve_for(wvar)
+                    c.solve()
                 n += 1
 
             self._marked_cons = []
@@ -625,7 +567,7 @@ class Solver(object):
             self._solving = False
 
 
-class solvable(object):
+class solvable:
     """
     Easy-to-use drop Variable descriptor.
 
@@ -640,15 +582,15 @@ class solvable(object):
     >>> a._v_x
     Variable(12, 20)
     >>> a.x = 3
-    >>> a.x 
+    >>> a.x
     Variable(3, 20)
-    >>> a.y 
+    >>> a.y
     Variable(0, 30)
     """
 
     def __init__(self, strength=NORMAL, varname=None):
         self._strength = strength
-        self._varname = varname or '_variable_%x' % id(self)
+        self._varname = varname or f"_variable_{id(self)}"
 
     def __get__(self, obj, class_=None):
         if not obj:
@@ -667,20 +609,18 @@ class solvable(object):
             setattr(obj, self._varname, v)
             v.value = value
 
-
+    def setvar(self, obj, v):
+        setattr(obj, self._varname, v)
 
 
 class JuggleError(AssertionError):
     """
-    Variable juggling exception. Raised when constraint's variables are
-    marking each other dirty forever.
+    Variable juggling exception. Raised when constraint's variables
+    are marking each other dirty forever.
     """
 
 
 __test__ = {
-    'Solver.add_constraint': Solver.add_constraint,
-    'Solver.remove_constraint': Solver.remove_constraint,
-    }
-
-
-# vim:sw=4:et:ai
+    "Solver.add_constraint": Solver.add_constraint,
+    "Solver.remove_constraint": Solver.remove_constraint,
+}
