@@ -1,18 +1,12 @@
 """
 Simple class containing the tree structure for the canvas items.
-
 """
 
-__version__ = "$Revision$"
-# $HeadURL$
 
-from operator import attrgetter
-
-
-class Tree(object):
+class Tree:
     """
     A Tree structure. Nodes are stores in a depth-first order.
-    
+
     ``None`` is the root node.
 
     @invariant: len(self._children) == len(self._nodes) + 1
@@ -24,10 +18,10 @@ class Tree(object):
         self._nodes = []
 
         # Per entry a list of children is maintained.
-        self._children = { None: [] }
+        self._children = {None: []}
 
         # For easy and fast lookups, also maintain a child -> parent mapping
-        self._parents = { }
+        self._parents = {}
 
     nodes = property(lambda s: list(s._nodes))
 
@@ -110,7 +104,7 @@ class Tree(object):
         siblings = self._children[parent]
         index = siblings.index(node) - 1
         if index < 0:
-            raise IndexError('list index out of range')
+            raise IndexError("list index out of range")
         return siblings[index]
 
     def get_all_children(self, node):
@@ -124,15 +118,14 @@ class Tree(object):
         >>> tree.get_children('n1')
         ['n2']
         >>> tree.get_all_children('n1') # doctest: +ELLIPSIS
-        <generator object get_all_children at 0x...>
+        <generator object Tree.get_all_children at 0x...>
         >>> list(tree.get_all_children('n1'))
         ['n2', 'n3']
         """
         children = self.get_children(node)
         for c in children:
             yield c
-            for cc in self.get_all_children(c):
-                yield cc
+            yield from self.get_all_children(c)
 
     def get_ancestors(self, node):
         """
@@ -145,7 +138,7 @@ class Tree(object):
         >>> tree.get_parent('n3')
         'n2'
         >>> tree.get_ancestors('n3') # doctest: +ELLIPSIS
-        <generator object get_ancestors at 0x...>
+        <generator object Tree.get_ancestors at 0x...>
         >>> list(tree.get_ancestors('n3'))
         ['n2', 'n1']
         >>> list(tree.get_ancestors('n1'))
@@ -158,8 +151,8 @@ class Tree(object):
 
     def index_nodes(self, index_key):
         """
-        Provide each item in the tree with an index attribute. This makes
-        for fast sorting of items.
+        Provide each item in the tree with an index attribute. This
+        makes for fast sorting of items.
 
         >>> class A(object):
         ...     def __init__(self, n):
@@ -176,43 +169,15 @@ class Tree(object):
         >>> t.index_nodes('my_key')
         >>> t.nodes[0].my_key, t.nodes[1].my_key, t.nodes[2].my_key
         (0, 1, 2)
-
-        For sorting, see ``sort()``.
         """
         nodes = self.nodes
         lnodes = len(nodes)
-        map(setattr, nodes, [index_key] * lnodes, xrange(lnodes))
-
-    def sort(self, nodes, index_key, reverse=False):
-        """
-        Sort a set (or list) of nodes.
-        
-        >>> class A(object):
-        ...     def __init__(self, n):
-        ...         self.n = n
-        ...     def __repr__(self):
-        ...         return self.n
-        >>> t = Tree()
-        >>> a = A('a')
-        >>> t.add(a)
-        >>> t.add(A('b'))
-        >>> t.add(A('c'), parent=a)
-        >>> t.nodes    # the series from Tree.index_nodes
-        [a, c, b]
-        >>> t.index_nodes('my_key')
-        >>> selection = (t.nodes[2], t.nodes[1])
-        >>> t.sort(selection, index_key='my_key')
-        [c, b]
-        """
-        if index_key:
-            return sorted(nodes, key=attrgetter(index_key), reverse=reverse)
-        else:
-            raise NotImplemented('index_key should be provided.')
+        list(map(setattr, nodes, [index_key] * lnodes, list(range(lnodes))))
 
     def _add_to_nodes(self, node, parent, index=None):
         """
-        Helper method to place nodes on the right location in the nodes list
-        Called only from add() and reparent()
+        Helper method to place nodes on the right location in the
+        nodes list Called only from add() and reparent()
         """
         nodes = self._nodes
         siblings = self._children[parent]
@@ -220,7 +185,7 @@ class Tree(object):
             atnode = siblings[index]
         except (TypeError, IndexError):
             index = len(siblings)
-            #self._add_to_nodes(node, parent)
+            # self._add_to_nodes(node, parent)
             if parent:
                 try:
                     next_uncle = self.get_next_sibling(parent)
@@ -236,7 +201,6 @@ class Tree(object):
         else:
             nodes.insert(nodes.index(atnode), node)
 
-
     def _add(self, node, parent=None, index=None):
         """
         Helper method for both add() and reparent().
@@ -246,7 +210,7 @@ class Tree(object):
         siblings = self._children[parent]
 
         self._add_to_nodes(node, parent, index)
-        
+
         # Fix parent-child and child-parent relationship
         try:
             siblings.insert(index, node)
@@ -257,18 +221,16 @@ class Tree(object):
         if parent:
             self._parents[node] = parent
 
-
     def add(self, node, parent=None, index=None):
         """
-        Add node to the tree. parent is the parent node, which may
-        be None if the item should be added to the root item.
+        Add node to the tree. parent is the parent node, which may be
+        None if the item should be added to the root item.
 
         For usage, see the unit tests.
         """
         self._add(node, parent, index)
         self._children[node] = []
 
-
     def _remove(self, node):
         # Remove from parent item
         self.get_siblings(node).remove(node)
@@ -294,19 +256,19 @@ class Tree(object):
     def _reparent_nodes(self, node, parent):
         """
         Helper for reparent().
-        The _children and _parent trees can be left intact as far as children
-        of the reparented node are concerned. Only the position in the
-        _nodes list changes.
+        The _children and _parent trees can be left intact as far as
+        children of the reparented node are concerned. Only the
+        position in the _nodes list changes.
         """
         self._nodes.remove(node)
         self._add_to_nodes(node, parent)
         for c in self._children[node]:
             self._reparent_nodes(c, node)
-        
+
     def reparent(self, node, parent, index=None):
         """
-        Set new parent for a ``node``. ``Parent`` can be ``None``, indicating
-        it's added to the top.
+        Set new parent for a ``node``. ``Parent`` can be ``None``,
+        indicating it's added to the top.
 
         >>> tree = Tree()
         >>> tree.add('n1')
@@ -323,7 +285,7 @@ class Tree(object):
         ['n1', 'n3', 'n2']
 
         If a node contains children, those are also moved:
-        
+
         >>> tree.add('n4')
         >>> tree.nodes
         ['n1', 'n3', 'n2', 'n4']
@@ -350,6 +312,3 @@ class Tree(object):
         # reorganize children in nodes list
         for c in self._children[node]:
             self._reparent_nodes(c, node)
-
-
-# vi: sw=4:et:ai
